LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_SIGNED.ALL;



ENTITY CD53_down IS
	generic(
		-- Bitwidth is the Bitwidth-1 of the internal arithmetic
		-- If Data is of size 8 BITWIDTH must be 7
		BITWIDTH : INTEGER := 15
		);
	PORT(
		enableN, clk, reset, clk_en		: IN	STD_LOGIC;
		data										: IN	STD_LOGIC_VECTOR(BITWIDTH downto 0);
		dout										: OUT	STD_LOGIC_VECTOR(BITWIDTH downto 0);
		cout										: OUT	STD_LOGIC_VECTOR(BITWIDTH downto 0);
		clk_en_out,enableOutN				: OUT	STD_LOGIC);
END CD53_down;




-- ====================================================================
-- This is a Wavelet Transform Core for the 5-3 Wavelet
-- 
-- boundary is treated automaticaly
--
-- The Arithmetik is scalable with the generic BITWIDTH
--
-- Vers. 1.0  27.09.01 D.Bachofen
-- ====================================================================


ARCHITECTURE	behavior OF CD53_down IS

--constant		BITWIDTH : INTEGER := 7;


		TYPE STATE_TYPE IS (s0, s1, s2, s3, s4, s5, s6, s7);
		SIGNAL	iInL1,InL1,InL2,InL3				: SIGNED(BITWIDTH downto 0);
		SIGNAL	iCalc1,CalcL1,CalcL2,CalcL3	: SIGNED(BITWIDTH downto 0);
		SIGNAL	add1									: SIGNED(BITWIDTH+1 downto 0);
		SIGNAL	sub1									: SIGNED(BITWIDTH downto 0);
		SIGNAL	add2									: SIGNED(BITWIDTH+1 downto 0);
		SIGNAL	sub2									: SIGNED(BITWIDTH downto 0);
		SIGNAL	dL1,iadd2							: SIGNED(BITWIDTH downto 0);
		SIGNAL	dout_int								: STD_LOGIC_VECTOR(BITWIDTH downto 0);
		SIGNAL	cout_int								: STD_LOGIC_VECTOR(BITWIDTH downto 0);
		SIGNAL	calcD1								: STD_LOGIC;
		SIGNAL	cnt1									: STATE_TYPE;
		SIGNAL	clk_en_out_set						: STD_LOGIC;

BEGIN

-- Hier sind die Eingangsregister InL1 - InL3 implementiert
	PROCESS (clk,clk_en)
	BEGIN 
		IF clk'EVENT AND clk='1' THEN
			-- clk enable wird ber Multiplexer am Eingang realisiert
			IF clk_en='1' THEN
				InL3 <= InL2;
				InL2 <= InL1;
				InL1 <= SIGNED(data);
			ELSE
				InL3 <= InL3 ;
				InL2 <= InL2 ;
				InL1 <= InL1 ;
			END IF;
		END IF;
	END PROCESS;


-- Umschalter fr Spezialbehandlung Zeilenende
	WITH enableN SELECT
		iCalc1<= 	InL1 	WHEN '0',
						InL3	WHEN OTHERS;


	
	
-- Generieren der Signale calcD1
	PROCESS (clk,clk_en,reset,cnt1 )
	BEGIN
		IF reset='1' THEN
			calcD1 <= '0';
		ELSIF clk'EVENT AND clk='0' THEN
			IF (cnt1>s2) AND  clk_en='1' THEN
				calcD1 <= not calcD1;
			ELSE
				calcD1 <= calcD1;
			END IF;
		END IF;
	END PROCESS;

-- Generieren der Signale clk_en_out
	PROCESS (calcD1,clk_en)
	BEGIN
		IF	calcD1='1'  AND clk_en='1' THEN
			clk_en_out_set <= '1';
		ELSE
			clk_en_out_set <= '0';
		END IF;
	END PROCESS;
			

	PROCESS (clk,clk_en,reset,calcD1 )
	BEGIN
		IF reset='1' THEN
			clk_en_out  <= '0';
		ELSIF clk'EVENT AND clk='0' THEN
			clk_en_out <= clk_en_out_set;
		END IF;
	END PROCESS;


-- Latch vor dem Berechnungsblock d1
	PROCESS (clk,clk_en,calcD1)
	BEGIN
		IF clk'EVENT AND clk='0' THEN
			IF (cnt1>s2) AND (calcD1='0') AND clk_en='1' THEN
				CalcL1 <= iCalc1;
				CalcL2 <= InL2;
				CalcL3 <= InL3;
			ELSE
				CalcL1 <= CalcL1 ;
				CalcL2 <= CalcL2;
				CalcL3 <= CalcL3;
			END IF;
		END IF;
	END PROCESS;


-- Latch vor den Ausgngen Cout und Dout
	PROCESS (clk,calcD1)
	BEGIN
		IF clk'EVENT AND clk='0' THEN
			IF calcD1 = '1' THEN
				cout_int   <= STD_LOGIC_VECTOR(sub2);			-- Cout wird hier gelatched
				dout_int   <= STD_LOGIC_VECTOR(sub1);			-- dout muss noch quantisiert werden
			ELSE
				cout_int <= cout_int;
				dout_int <= dout_int;
			END IF;
		END IF;
	END PROCESS;
	cout <= cout_int;
	dout <= dout_int;


-- Arithmetik fr dout1 (HP) mit steigender Dynamik
--	add1 <= (CalcL1(BITWIDTH) & CalcL1) + (CalcL3(BITWIDTH) & CalcL3);
--	sub1 <= (add1(BITWIDTH+1)&add1(BITWIDTH+1 downto 1)) - (CalcL2(BITWIDTH) & CalcL2) ; 	-- Subtraktion mit vorgngigem rechtsshift

-- Arithmetik fr dout1 (HP)
	add1 <= (CalcL1(BITWIDTH) & CalcL1) + (CalcL3(BITWIDTH) & CalcL3);
	sub1 <= add1(BITWIDTH+1 downto 1) - CalcL2 ; 	-- Subtraktion mit vorgngigem rechtsshift



-- AUSGANG dout und Latch dL1
	PROCESS (clk,calcD1)
	BEGIN
		IF clk'EVENT AND clk='1' THEN
			IF (calcD1='0') THEN
				dL1 <= sub1;
			END IF;
		END IF;
	END PROCESS;

-- Umschalter fr Spezialbehandlung Zeilenanfang
	WITH cnt1 SELECT
		iadd2 <= sub1 WHEN s2 | s3 | s4,
				 	dL1  WHEN OTHERS;

-- Arithmetik fr cout (TP) mit steigender Dynamik
--	add2 <= (iadd2(BITWIDTH+1) & iadd2) + (sub1(BITWIDTH+1) & sub1);
--	sub2 <= (CalcL3(BITWIDTH) & CalcL3(BITWIDTH) & CalcL3) - (add2(BITWIDTH+2) & add2(BITWIDTH+2) & add2(BITWIDTH+2 downto 2));

-- Arithmetik fr cout (TP)
	add2 <= (iadd2(BITWIDTH) & iadd2) + (sub1(BITWIDTH) & sub1);
	sub2 <= CalcL3 - (add2(BITWIDTH+1) & add2(BITWIDTH+1 downto 2));



-- enableOutN
	PROCESS (clk,cnt1,reset)
	BEGIN
		IF reset='1' THEN
			enableOutN<='1';
		ELSIF clk'EVENT AND clk='1' THEN
			IF cnt1=s3 THEN
				enableOutN<='0';
			ELSIF (cnt1=s0)	THEN
				enableOutN<='1';
			END IF;
		END IF;
	END PROCESS;


-- Zhler cnt1 zur Statusanzeige
-- Statemachine fr cnt1 ( wenn vertSync=0 s0-s1-s2-s3-s4 dann
--                         wenn vertSync=1 s5-s6-s0 
	PROCESS (clk,reset,clk_en,enableN)
	BEGIN
		IF reset = '1' THEN
			cnt1 <= s0;
		ELSIF clk'EVENT AND clk='1' THEN
			CASE cnt1 IS
				WHEN s0 =>
					IF (enableN = '0') AND (clk_en = '1') THEN
						cnt1 <= s1;
					ELSE
						cnt1 <= s0;
					END IF;
				WHEN s1 =>
					IF (clk_en = '1') THEN
						cnt1 <= s2;
					ELSE
						cnt1 <= s1;
					END IF;
				WHEN s2 =>
					IF (clk_en = '1') THEN
						cnt1 <= s3;
					ELSE
						cnt1 <= s2;
					END IF;
				WHEN s3 =>
					IF (clk_en = '1') THEN
						cnt1 <= s4;
					ELSE
						cnt1 <= s3;
					END IF;
				WHEN s4 =>
					IF (clk_en = '1') THEN
						cnt1 <= s5;
					ELSE
						cnt1 <= s4;
					END IF;
				WHEN s5 =>
					IF (enableN = '1') and (clk_en = '1') THEN
						cnt1 <= s6;
					ELSE
						cnt1 <= s5;
					END IF;
				WHEN s6 =>
					IF (clk_en = '1') THEN
						cnt1 <= s7;
					ELSE
						cnt1 <= s6;
					END IF;
				WHEN s7 =>
					IF (clk_en = '1') THEN
						cnt1 <= s0;
					ELSE
						cnt1 <= s7;
					END IF;
			END CASE;
		END IF;
	END PROCESS;



END behavior;
