library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity FrameLoader is
    Port ( reset : in std_logic;								--asynch Reset (active high)
			  trig : in std_logic;								--start grabbing (active high)
	 		  llck : in std_logic;								--Clock from SAA7113
           vpo : in std_logic_vector(7 downto 0);		--Data from SAA7113
           rts : in std_logic_vector(1 downto 0);		--realtime video status
           ce : out std_logic;								--chip enable to video ram (active low)
           oe : out std_logic;								--outp enable to video ram (active low)
           we : out std_logic;								--write enable to video ram (active low)
           adr : out std_logic_vector(18 downto 0);	--address into video ram
           data : out std_logic_vector(15 downto 0);	--data into video ram
			  dataReady: out std_logic );						--High when entire fream read
end FrameLoader;

architecture behavioral of FrameLoader is

constant NO  : std_logic := '0';
constant YES : std_logic := '1';
constant LO  : std_logic := '0';
constant HI  : std_logic := '1';



-- states for the SAA7113 interface circuit	
type video_state is (
	wait_for_vert_sync,
	vert_sync,
	odd_frame_read,
	wait_for_horiz_sync,
	horiz_sync,
	active_video_chroma_b,
	active_video_lum_b,
	active_video_chroma_r,
	active_video_lum_r,
	display
);

--signal adr_cntr : std_logic_vector(17 downto 0);
signal line_cntr : std_logic_vector(8 downto 0);
signal pix_cntr : std_logic_vector(9 downto 0);
signal adr_int : std_logic_vector(18 downto 0);
signal store_data : std_logic;
signal odd_read : std_logic;
signal dataReadyInt : std_logic;
signal vid_next	: video_state;		-- state of SAA7113 interface


begin

-----------------------------------------------------------------------------------------
--         Statemachine um videosignal zu lesen
-----------------------------------------------------------------------------------------
	process(llck,reset)
	begin

		if(reset=YES) then
			vid_next <= display;
			data(15 downto 8) <= "11111111";
			dataReadyInt <= YES;
			store_data <= NO;
			odd_read <= NO;

		-- drive state machine with video clock from SAA7113
		elsif(llck'event and llck=HI) then --llck

			store_data <= NO;
			dataReadyInt <= NO;

			case vid_next is

			when wait_for_vert_sync =>
				data(15 downto 8) <= "00000001";
				line_cntr <= (others=>'1'); -- auf FFFF damit nachher null
				if(rts(1)=LO) then
					vid_next <= vert_sync;
				end if;

			when vert_sync =>
				data(15 downto 8) <= "00000010";
				odd_read<=NO;
--				adr_cntr <= (others=>'0');
				if(rts(1)=HI) then
					vid_next <= wait_for_horiz_sync;
				end if;

			when wait_for_horiz_sync =>
				data(15 downto 8) <= "00000011";
				if(rts(0)=LO) then
					vid_next <= horiz_sync;
				end if;

			when horiz_sync =>
				data(15 downto 8) <= "00000101";
				if(rts(0)=HI) then
					pix_cntr <= (others=>'0');
					line_cntr <= line_cntr+1;
					vid_next <= active_video_lum_b;
				end if;

			when active_video_chroma_b =>	-- chroma B data from video stream
				data(15 downto 8) <= "00000110";
				if((rts(1)=HI) and (odd_read=YES)) then
					vid_next <= display;
				elsif ((rts(1)=LO) and (odd_read=NO)) then
					vid_next <= odd_frame_read;
				elsif(rts(0)=LO) then
					vid_next <= horiz_sync;
				else
					vid_next <= active_video_lum_b;
				end if;

			when active_video_lum_b =>	-- latch luminance data from video stream
				data(15 downto 8) <= "00000111";
				if((rts(1)=HI) and (odd_read=YES)) then
					vid_next <= display;
				elsif ((rts(1)=LO) and (odd_read=NO)) then
					vid_next <= odd_frame_read;
				elsif(rts(0)=LO) then
					vid_next <= horiz_sync;
				else
					vid_next <= active_video_chroma_r;
					data(7 downto 0) <= vpo;
					store_data <= YES;
					pix_cntr <= pix_cntr + 1;
					adr <= adr_int;
				end if;

			when active_video_chroma_r =>	-- chroma R data from video stream
				data(15 downto 8) <= "00001000";
				if((rts(1)=HI) and (odd_read=YES)) then
					vid_next <= display;
				elsif ((rts(1)=LO) and (odd_read=NO)) then
					vid_next <= odd_frame_read;
				elsif(rts(0)=LO) then
					vid_next <= horiz_sync;
				else
					vid_next <= active_video_lum_r;
				end if;

			when active_video_lum_r =>	-- latch luminance data from video stream
				data(15 downto 8) <= "00001001";
				if((rts(1)=HI) and (odd_read=YES)) then
					vid_next <= display;
				elsif ((rts(1)=LO) and (odd_read=NO)) then
					vid_next <= odd_frame_read;
				elsif(rts(0)=LO) then
					vid_next <= horiz_sync;
				else
					vid_next <= active_video_chroma_b;
					data(7 downto 0) <= vpo;
					store_data <= YES;
					pix_cntr <= pix_cntr + 1;
					adr <= adr_int;
				end if;
				

			when odd_frame_read =>
				data(15 downto 8) <= "10011001";
				odd_read <= YES;
				vid_next <= horiz_sync;
				line_cntr <= (others=>'1'); -- auf FFFF damit nachher null
				

			when display =>	-- display stored frame data
				data(15 downto 8) <= "00001111";
				dataReadyInt <= YES;
				if(trig=HI) then
					vid_next <= wait_for_vert_sync;
				end if;

			when others =>	-- handle erroneous states
				vid_next <= wait_for_vert_sync;

			end case;

		end if;

	end process;

	-- create write enable, output enable and chip enable signals:
--	we <= LO when ((store_data=YES) and (llck=HI))  else HI;   --
	process (llck)
	begin
		if llck'event and llck='0' then
			we<=not store_data;
		end if;
	end process;
	
	adr_int <= line_cntr(7 downto 0) & (not odd_read) & pix_cntr;
	oe <= LO when (dataReadyInt=YES) else HI;
	ce <= line_cntr(8);
	dataReady <= dataReadyInt;


end behavioral;
