library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity USB_receiver is
	generic
	(
		-- CLK_DIV is the clock divider
		-- the input clk is divided by CLK_DIV+1
		-- CLK_DIV must be a 6 digit No
		-- CLK_DIV must >= 4
		CLK_DIV: std_logic_vector(5 downto 0) := "011111"	-- 31
	);
   Port ( 
	 			reset 		: in std_logic;
				clk 			: in std_logic;								-- should be 50MHz
				RCV			: in 	std_logic;
				VP				: in 	std_logic;
				VM				: in 	std_logic;
				data 			: out std_logic_vector(7 downto 0);
				bitwidth 	: out std_logic_vector(2 downto 0);		-- if 8Bits received bitwidth is 7
				data_ready 	: out std_logic;
				VPO 			: out std_logic;
				FSE 			: out std_logic;
				SUSPND		: out std_logic;
				OEn			: out std_logic);
end USB_receiver;


-- This Entity is a seriel port using USB Transceivers but not using
-- USB standard comunication protocol. USB Transceiver should be set
-- to high speed
--
-- !!! Do not use this Entity with other standard USB Components !!!!
--
-- The signals VPO and FSE are not used because this is only a 
-- receiver. Leave this signals with a Pullup
--
-- The data is received with MSB first
-- The output is not latched and must be latched externaly when
-- data_ready is high


architecture behavioral of USB_receiver is



-- ____________________________________________________________________
-- ====================================================================
-- Constants:
-- ====================================================================

constant HI : std_logic 	:= '1';
constant LO : std_logic 	:= '0';
constant YES : std_logic 	:= '1';
constant NO : std_logic 	:= '0';





type STATE_TYPE is (
			IDLE,
			RECEIVING,
			LATCH_OUTPUT
	);



-- ____________________________________________________________________
-- ====================================================================
-- Signals:
-- ====================================================================
signal cState,nState :	STATE_TYPE;
signal clk_cnt	: 			std_logic_vector(5 downto 0);		-- counts the clk to divide the clk
signal res_clk_cnt : 	std_logic;	-- resets the clk counter
signal reset_int :		std_logic;	-- resets the counters and shift reg during IDLE state
signal next_bit : 		std_logic;	-- signals to send next bit
signal read_bit : 		std_logic;	-- signals to read bit
signal bit_cnt	: 			std_logic_vector(2 downto 0);		-- counts the bits sent
signal shftReg:			std_logic_vector(7 downto 0);
signal SE0 : 				std_logic;	-- signals when a single ended zero is forced
signal data_ready_int : std_logic;	-- HI during LATCH_OUTPUT state



begin


	SUSPND <= LO;		-- work always
	OEn <= HI;			-- work as receiver
	VPO <= LO;
	FSE <= LO;

	
	process (clk, VP, VM)
	begin
		if clk'event and clk=LO then
			if ((VP=LO) and (VM=LO))then
				SE0 <= HI;
			else
				SE0 <= LO;
			end if;
		end if;
	end process;




-- ____________________________________________________________________
-- ====================================================================
-- Counter for dividing the clk
-- ====================================================================
	process(clk, res_clk_cnt, reset_int)
	begin
		if clk'event and clk=HI then
			if res_clk_cnt=HI then
				clk_cnt <= (others => '0');
			elsif reset_int = LO then
				clk_cnt <= clk_cnt + 1;
			end if;
		end if;
	end process;

	-- Reset for the counter
	process (next_bit, reset_int)
	begin
		if (next_bit=HI or reset_int=HI) then
			res_clk_cnt <= HI;
		else
			res_clk_cnt <= LO;
		end if;
	end process;


-----------------------------------------------------------------------
-- Signal for clk_cnt = CLK_DIV
-----------------------------------------------------------------------
	process(clk_cnt)
	begin
		if clk_cnt = CLK_DIV then
			next_bit <= HI;
		else
			next_bit <= LO;
		end if;
	end process;

-----------------------------------------------------------------------
-- Signal for reading bit
-----------------------------------------------------------------------
	process(clk_cnt)
	begin
		-- count to floor(CLK_DIV/2)
		if clk_cnt = ("0" & CLK_DIV(5 downto 1)) then
			read_bit <= HI;
		else
			read_bit <= LO;
		end if;
	end process;



-- ____________________________________________________________________
-- ====================================================================
-- Counter for Bitwidth
-- ====================================================================
	process(clk, reset_int, read_bit)
	begin
		if clk'event and clk=HI then
			if reset_int = HI then
				-- set to "111" so if 1 bit is received
				-- bt_cnt is "000"
				bit_cnt <= (others => '1');
			elsif read_bit=HI then
				bit_cnt <= bit_cnt + 1;
			end if;
		end if;
	end process;

	bitwidth <= bit_cnt;
				



	
-- ____________________________________________________________________
-- ====================================================================
-- ShiftRegister
-- ====================================================================
	process(clk, reset_int, read_bit)
	begin
		if clk'event and clk=HI then
			if reset_int=HI then
				shftReg <= (others => '0');
			elsif read_bit = HI then
				shftReg <= (shftReg(6 downto 0) & RCV);
			end if;
		end if;
	end process;

	data <= shftReg;

	

-- ____________________________________________________________________
-- ====================================================================
-- data_ready signal
-- ====================================================================
	process(clk)
	begin
		if clk'event and clk=LO then
			data_ready <= data_ready_int;
		end if;
  	end process;


-- ____________________________________________________________________
-- ====================================================================
-- State machine
-- ====================================================================
	process(clk,reset)
	begin
		if reset=HI then
			cState <= IDLE;
		elsif clk'event and clk=HI then
			cState <= nState;
		end if;
	end process;


	process(cState, SE0, next_bit)
	begin
		reset_int <= LO;
		data_ready_int <= LO;

		case cState is
			when IDLE =>
				reset_int <= HI;
				if SE0 = LO then
					nState <= RECEIVING;
				else
					nState <= IDLE;
				end if;

			when RECEIVING =>
				if ((SE0 = HI)) then --and (next_bit = HI)) then
					nState <= LATCH_OUTPUT;
				else
					nState <= RECEIVING;
				end if;
					
				
			when LATCH_OUTPUT =>
				data_ready_int <= HI;
				nState <= IDLE;
		
			when others =>
				nState <= IDLE;
			end case;
	end process;

end behavioral;
