library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity USB_transmitter is
	generic
	(
		-- CLK_DIV is the clock divider
		-- the input clk is divided by CLK_DIV+1
		-- CLK_DIV must be a 6 digit No
		-- CLK_DIV must >= 4
		CLK_DIV: std_logic_vector(5 downto 0) := "011111"	-- 31
	);
   Port ( 
	 			reset 		: in std_logic;
				clk 			: in std_logic;								-- should be 50MHz
				data 			: in std_logic_vector(7 downto 0);
				bitwidth 	: in std_logic_vector(2 downto 0);		-- if sending 8Bits bitwidth is 7
				send_data 	: in std_logic;
				VPO 			: out std_logic;
				FSE 			: out std_logic;
				SUSPND		: out std_logic;
				OEn			: out std_logic;
				busy 			: out std_logic);
end USB_transmitter;


-- This Entity is a seriel port using USB Transceivers but not using
-- USB standard comunication protocol USB Transceiver should be set
-- to high speed
--
-- !!! Do not use this Entity with other standard USB Components !!!!
--
-- The signals RCV, VP and VM are not used because this is only a 
-- transmitter. Leave this signals with a Pullup
--
-- The data is sent with MSB first


architecture behavioral of USB_transmitter is



-- ____________________________________________________________________
-- ====================================================================
-- Constants:
-- ====================================================================

constant HI : std_logic 	:= '1';
constant LO : std_logic 	:= '0';
constant YES : std_logic 	:= '1';
constant NO : std_logic 	:= '0';





type STATE_TYPE is (
			IDLE,
			SENDING,
			END1,
			END2,
			END3,
			END4
	);



-- ____________________________________________________________________
-- ====================================================================
-- Signals:
-- ====================================================================
signal cState,nState :	STATE_TYPE;
signal clk_cnt	: 			std_logic_vector(5 downto 0);		-- counts the clk to divide the clk
signal res_clk_cnt : 	std_logic;	-- resets the clk counter
signal load : 				std_logic;	-- load is high during IDLE state
signal next_bit : 		std_logic;	-- signals to send next bit
signal bit_cnt	: 			std_logic_vector(2 downto 0);		-- counts the bits sent
signal data_sent:			std_logic;	-- Hi when all bits sent
signal shftReg:			std_logic_vector(7 downto 0);


begin


	SUSPND <= LO;	-- work always
	OEn <= LO;		-- work as transmitter

-- ____________________________________________________________________
-- ====================================================================
-- Counter for dividing the clk
-- ====================================================================
	process(clk, res_clk_cnt, load)
	begin
		if clk'event and clk=HI then
			if res_clk_cnt=HI then
				clk_cnt <= (others => '0');
			elsif load = LO then
				clk_cnt <= clk_cnt + 1;
			end if;
		end if;
	end process;

	-- Reset for the counter
	process (next_bit, load)
	begin
		if next_bit=HI or load=HI then
			res_clk_cnt <= HI;
		else
			res_clk_cnt <= LO;
		end if;
	end process;


-----------------------------------------------------------------------
-- Signal for clk_cnt = CLK_DIV
-----------------------------------------------------------------------
	process(clk_cnt)
	begin
		if clk_cnt = CLK_DIV then
			next_bit <= HI;
		else
			next_bit <= LO;
		end if;
	end process;



-- ____________________________________________________________________
-- ====================================================================
-- Counter for Bitwidth
-- ====================================================================
	process(clk, load, next_bit)
	begin
		if clk'event and clk=HI then
			if load = HI then
				bit_cnt <= bitwidth;
			elsif next_bit=HI then
				bit_cnt <= bit_cnt - 1;
			end if;
		end if;
	end process;

-----------------------------------------------------------------------
-- Signal for data sent
-----------------------------------------------------------------------
	process(bit_cnt)
	begin
		if bit_cnt="000" then
			data_sent <= HI;
		else
			data_sent <= LO;
		end if;
	end process;

	
-- ____________________________________________________________________
-- ====================================================================
-- ShiftRegister
-- ====================================================================
	process(clk, load, next_bit)
	begin
		if clk'event and clk=HI then
			if load=HI then
				shftReg <= data;
			elsif next_bit = HI then
				shftReg <= (shftReg(6 downto 0) & "0");
			end if;
		end if;
	end process;
	
	-- Send the MSB to the USB transceiver
  	VPO <= shftReg(7);



-- ____________________________________________________________________
-- ====================================================================
-- State machine
-- ====================================================================
	process(clk,reset)
	begin
		if reset=HI then
			cState <= IDLE;
		elsif clk'event and clk=HI then
			cState <= nState;
		end if;
	end process;


	process(cState, send_data, data_sent, next_bit)
	begin
		busy <= HI;
		FSE <= HI;
		load <= LO;

		case cState is
			when IDLE =>
				busy <= LO;
				load <= HI;
				if send_data = HI then
					nState <= SENDING;
				else
					nState <= IDLE;
				end if;

			when SENDING =>
				FSE <= LO;
				if data_sent=HI and next_bit=HI then
					nState <= END1;
				else
					nState <= SENDING;
				end if;
					
				
			when END1 =>
				nState <= END2;

			when END2 =>
				load <= HI;
				nState <= END3;

			when END3 =>
				load <= HI;
				nState <= END4;

			when END4 =>
				load <= HI;
				nState <= IDLE;
		
			when others =>
				nState <= IDLE;
			end case;
	end process;

end behavioral;
