library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;


--
-- component that initializes the colormap of the BT481 RAMDAC
--
entity bt481_init is
port(
	reset		: in std_logic;
	clk			: in std_logic;
	load_color	: in std_logic;		-- HI=load colors, LO=load grayscale into colormap
	wrb			: out std_logic;	-- write-strobe to BT481
	rdb			: out std_logic;	-- read-strobe to BT481
	d			: out std_logic_vector(7 downto 0);	-- data to BT481
	rs			: out std_logic_vector(2 downto 0)	-- register select for BT481
);
end bt481_init;



architecture bt481_init_arch of bt481_init is

constant NO		: std_logic := '0';
constant YES	: std_logic := '1';
constant LO		: std_logic := '0';
constant HI		: std_logic := '1';

-- register addresses for the BT481
constant PALETTE_WRITE_ADDRESS_REG	: std_logic_vector(2 downto 0) := "000";
constant PALETTE_READ_ADDRESS_REG	: std_logic_vector(2 downto 0) := "011";
constant PALETTE_RAM_REG			: std_logic_vector(2 downto 0) := "001";
constant PIXEL_MASK_REG				: std_logic_vector(2 downto 0) := "010";
constant OVERLAY_WRITE_ADDRESS_REG	: std_logic_vector(2 downto 0) := "100";
constant OVERLAY_READ_ADDRESS_REG	: std_logic_vector(2 downto 0) := "111";
constant OVERLAY_RAM_REG			: std_logic_vector(2 downto 0) := "101";
constant COMMAND_A_REG				: std_logic_vector(2 downto 0) := "110";

constant PALETTE_SIZE 	: integer := 256;	-- BT481 has 256 entries in colormap
constant END_ADDRESS	: integer := PALETTE_SIZE-1;

-- state definitions for BT481 initialization state machine
type init_state is (
	load_pixel_mask,
	load_cmd_A,
	load_palette_address,
	load_palette_red,
	load_palette_green,
	load_palette_blue,
	done
);

signal init_next	: init_state;	-- state of initialization state machine
signal palette_address : std_logic_vector(7 downto 0);
signal wr			: std_logic;	-- internal write signal
signal gray_cnt		: std_logic_vector(1 downto 0);	-- gray-code counter
signal init_clk		: std_logic;	-- internal clock for state machine

begin

	-- divide input clock to 1/4 frequency
	process(clk)
	begin
		if(clk'event and clk=HI) then
			case(gray_cnt) is
			when "00" => gray_cnt <= "01";
			when "01" => gray_cnt <= "11";
			when "11" => gray_cnt <= "10";
			when "10" => gray_cnt <= "00";
			when others => gray_cnt <= "00";
			end case;
		end if;
	end process;
	init_clk <= gray_cnt(1);

	process(reset,init_clk)
	begin

		if(reset=YES) then
			palette_address <= (others=>'0');	-- load colormap starting from address 0
			wr <= NO;
			init_next <= load_pixel_mask;

		elsif(init_clk'event and init_clk=HI) then

			case(init_next) is

			when load_pixel_mask =>	-- load pixel data mask
				wr <= YES;
				rs <= PIXEL_MASK_REG;
				d <= "11111111";	-- don't mask any bits
				init_next <= load_cmd_A;

			when load_cmd_A =>	-- load command register A to enable colormap
				wr <= YES;
				rs <= COMMAND_A_REG;
				d <= "00000000";	-- default command value enables colormap
				init_next <= load_palette_address;

			when load_palette_address =>	-- load address of colormap entry
				wr <= YES;
				rs <= PALETTE_WRITE_ADDRESS_REG;
				d <= palette_address;
				init_next <= load_palette_red;

			when load_palette_red =>	-- load red component of colormap entry
				wr <= YES;
				rs <= PALETTE_RAM_REG;
				if(load_color=YES) then	-- load colorful colormap
					-- red component comes from upper 3 bits of palette address
					d <= palette_address(7 downto 5) & "00000";
				else	-- load grayscale colormap
					-- gray level comes from palette address so initial entries
					-- are darker and later entries are lighter
					d <= palette_address;
				end if;
				init_next <= load_palette_green;

			when load_palette_green =>	-- load green component of colormap entry
				wr <= YES;
				rs <= PALETTE_RAM_REG;
				if(load_color=YES) then	-- load colorful colormap
					-- green component comes from middle 2 bits of palette address
					d <= palette_address(4 downto 3) & "000000";
				else	-- load grayscale colormap
					-- gray level comes from palette address
					d <= palette_address;
				end if;
				init_next <= load_palette_blue;

			when load_palette_blue =>	-- load blue component of colormap entry
				wr <= YES;
				rs <= PALETTE_RAM_REG;
				if(load_color=YES) then	-- load colorful colormap
					-- blue component comes from lower 3 bits of palette address
					d <= palette_address(2 downto 0) & "00000";
				else	-- load grayscale colormap
					-- gray level comes from palette address
					d <= palette_address;
				end if;
				palette_address <= palette_address + 1;
				if(palette_address=END_ADDRESS) then
					init_next <= done;
				else
					init_next <= load_palette_address;
				end if;

			when done =>	-- colormap has been loaded
				wr <= NO;
				init_next <= done;

			when others =>	-- handle erroneous states
				init_next <= load_pixel_mask;

			end case;

		end if;

	end process;

	-- pulse the write strobe during the third quarter of each state machine cycle.
	-- This gives the BT481 a stable register address and data value with ample
	-- setup and hold times.
	wrb <= '0' when wr=YES and gray_cnt="00" else '1';
	rdb <= YES;	-- disable reading from BT481

end bt481_init_arch;

