LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;

ENTITY bt481_vga IS
GENERIC
(
	CLK_FREQ: INTEGER := 25_000	-- input clock in KHz
);
PORT
(
	clk:	IN STD_LOGIC;		-- VGA dot clock
	reset:	IN STD_LOGIC;		-- asynchronous reset
	hsyncb: OUT STD_LOGIC;		-- horizontal (line) sync
	vsyncb: OUT STD_LOGIC;		-- vertical (frame) sync
	pixel:	OUT STD_LOGIC_VECTOR(7 DOWNTO 0);	-- red,green,blue colors
	address:OUT STD_LOGIC_VECTOR(18 DOWNTO 0);	-- address into video RAM
	data:	IN STD_LOGIC_VECTOR(7 DOWNTO 0);	-- data from video RAM
	ceb:	OUT STD_LOGIC;		-- video RAM chip enable
	oeb:	OUT STD_LOGIC;		-- video RAM output enable
	web:	OUT STD_LOGIC;		-- video RAM write enable
	blankb:	OUT STD_LOGIC		-- video blanking
);
END bt481_vga;

ARCHITECTURE bt481_vga_arch OF bt481_vga IS


-- Horizontal timing
CONSTANT H_PERIOD:		INTEGER := INTEGER((CLK_FREQ * 31770) / 1E6 + 1);	-- horizontal video line period
--                =795 @ 25MHz

CONSTANT HVIS_INTERVAL: INTEGER := 640;	-- visible horizontal video interval
CONSTANT H_OFFSET: INTEGER := 21; -- RAM Adr. Offset (21)
CONSTANT HSYNC_START:	INTEGER := INTEGER((CLK_FREQ * 26110) / 1E6 + 1 );	-- time when horiz. sync pulse starts
--                =653 @ 25MHz

CONSTANT HSYNC_STOP:	INTEGER := INTEGER((CLK_FREQ * 29880) / 1E6 + 1 );	-- time when horiz. sync pulse ends
--                =748 @ 25MHz


-- Vertical Timing	
CONSTANT V_PERIOD:		INTEGER := INTEGER((CLK_FREQ * 16784) / (1E3 * H_PERIOD) + 1);	-- vertical video frame period
--                =528 @ 25MHz

CONSTANT VVIS_INTERVAL:	INTEGER := 480;	-- visible vertical video interval
CONSTANT V_OFFSET: INTEGER := 32; -- RAM Adr. Offset (32)
CONSTANT VSYNC_START:	INTEGER := INTEGER((CLK_FREQ * 15700) / (1E3 * H_PERIOD) + 1 );	-- time when vert. sync pulse starts
--                =494 @ 25MHz

CONSTANT VSYNC_STOP:	INTEGER := INTEGER((CLK_FREQ * 15764) / (1E3 * H_PERIOD) + 1 );	-- time when vert. sync pulse ends
--                =496 @ 25MHz
	
SIGNAL low_adr,hcnt: STD_LOGIC_VECTOR(9 DOWNTO 0);	-- horizontal pixel counter
SIGNAL hi_adr,vcnt: STD_LOGIC_VECTOR(9 DOWNTO 0);	-- vertical line counter
SIGNAL pixrg: STD_LOGIC_VECTOR(7 DOWNTO 0);	-- byte-wide register for 4 pixels
SIGNAL int_blank: STD_LOGIC;				-- video blanking signal
SIGNAL pblank: STD_LOGIC;					-- pipelined video blanking signal
SIGNAL int_hsyncb: STD_LOGIC;				-- internal horizontal sync.
signal evenFrame : std_logic;

BEGIN

	inc_horiz_pixel_counter:
	PROCESS(clk,reset)
	BEGIN
		IF reset='1' THEN -- reset asynchronously clears pixel counter
			hcnt <= (others=>'0');
		ELSIF (clk'EVENT AND clk='1') THEN
			IF hcnt < H_PERIOD THEN -- horiz. pixel counter rolls-over after line of pixels
				hcnt <= hcnt + 1;
			ELSE
				hcnt <= (others=>'0');
			END IF;
		END IF;
	END PROCESS;

	inc_vert_line_counter:
	PROCESS(int_hsyncb,reset)
	BEGIN
		IF reset='1' THEN -- reset asynchronously clears line counter
			vcnt <= (others=>'0');
			evenFrame <= '0';
		ELSIF (int_hsyncb'EVENT AND int_hsyncb='1') THEN
			IF vcnt<(V_PERIOD) THEN -- vert. line counter rolls-over after frame of lines
				vcnt <= vcnt + 1;
			ELSE
				vcnt <= (others=>'0');
				evenFrame <= not evenFrame;
			END IF;
		END IF;
	END PROCESS;

	generate_horiz_sync:
	PROCESS(clk,reset)
	BEGIN
		IF reset='1' THEN -- reset asynchronously sets horizontal sync to inactive
			int_hsyncb <= '1';
		ELSIF (clk'EVENT AND clk='1') THEN
			if(hcnt=HSYNC_START and int_hsyncb='1') then
				int_hsyncb <= '0';
			end if;
			if(hcnt=HSYNC_STOP and int_hsyncb='0') then
				int_hsyncb <= '1';
			end if;

--			IF (hcnt>=HSYNC_START AND hcnt<HSYNC_STOP) THEN
--				-- horiz. sync is low in this interval to signal start of a new line
--				int_hsyncb <= '0';
--			ELSE
--				int_hsyncb <= '1';
--			END IF;
		END IF;
	END PROCESS;
	hsyncb <= int_hsyncb;	-- output the horizontal sync signal

	generate_vert_sync:
	PROCESS(int_hsyncb,reset)
	BEGIN
		IF reset='1' THEN -- reset asynchronously sets vertical sync to inactive
			vsyncb <= '1';
		-- vertical sync is recomputed at the end of every line of pixels
		ELSIF (int_hsyncb'EVENT AND int_hsyncb='1') THEN
			IF (vcnt>=VSYNC_START AND vcnt<VSYNC_STOP) THEN
				-- vert. sync is low in this interval to signal start of a new frame
				vsyncb <= '0';
			ELSE
				vsyncb <= '1';
			END IF;
		END IF;
	END PROCESS;

	-- blank video outside of visible region: (0,0) -> (255,479)
	int_blank <= '1' WHEN (hcnt>=(HVIS_INTERVAL) OR vcnt>=(VVIS_INTERVAL)) ELSE '0';
	-- store the blanking signal for use in the next pipeline stage
	pipeline_blank:
	PROCESS(clk,reset)
	BEGIN
		IF reset='1' THEN
			pblank <= '0';
		ELSIF (clk'EVENT AND clk='1') THEN
			pblank <= int_blank;
		END IF;
	END PROCESS;

	-- video RAM control signals
	ceb <= '0';		-- enable the RAM
	web <= '1';		-- disable writing to the RAM
	oeb <= int_blank;	-- enable the RAM outputs when video is not blanked

	-- The video RAM address is built from the lower 9 bits of the vertical
	-- line counter and the lower 8 bits of the horizontal pixel counter.
	-- Each byte of the RAM contains one 8-bit pixel.
	hi_adr<=vcnt+V_OFFSET;
	low_adr<=hcnt+H_OFFSET;
	address <= hi_adr(8 DOWNTO 0) & low_adr(9 DOWNTO 0);

	update_pixel_register:
	PROCESS(clk,reset)
	BEGIN
		IF reset='1' THEN -- clear the pixel register on reset
			pixrg <= "00000000";
		ELSIF (clk'EVENT AND clk='1') THEN
			pixrg <= data;	-- load pixel register with byte from RAM
		END IF;
	END PROCESS;

	-- output the blanking and pixel data
	blankb	<= not pblank;
	pixel	<= pixrg when (hi_adr(0)=evenFrame) else (others => '0');

END bt481_vga_arch;

