library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_SIGNED.ALL;


entity cd53_up_8Bit is
	Port ( 	reset 			: in std_logic;	-- active HI reset
				clk 				: in std_logic;
				enablen_in 		: in std_logic;	-- active LO enable
				clk_en_in 		: in std_logic;
				data_in 			: in std_logic_vector(7 downto 0);
				data_out 		: out std_logic_vector(7 downto 0);
				clk_en_out 		: out std_logic;
				enablen_out 	: out std_logic);	-- active LO enable out
end cd53_up_8Bit;

-- ====================================================================
-- Inverse Wavelet Transformation Arithmetic for 5-3 Wavelet
-- ====================================================================
--
-- Arithmetic for Inverse 5-3 Wavelet Transform. Boundarys are treated
-- automatically.
-- At the beginning of a line of data set the enable to low. At
-- the end set the enable to high. set clk enable high of data is ready
-- Send alternating Aprox. (c) and Detailcoefs to the core (beginning with
-- Aprox. Coeffs)
-- At the output you get a datastream with the original data
--
-- USE Xilinx XST Technology to Synthesize; FPGA Express does not
-- generate apropriate results
--
-- 
-- The Arithmetic is set to 8 Bit
--
-- Vers. 1.0  27.09.01 D.Bachofen
-- ====================================================================

architecture behavioral of cd53_up_8Bit is

-- ====================================================================
-- Architecture declarations
-- ====================================================================


-----------------------------------------------------------------------
-- Constants:
-----------------------------------------------------------------------
constant HI 	: std_logic := '1';
constant LO 	: std_logic := '0';
constant YES 	: std_logic := '1';
constant NO 	: std_logic := '0';
constant TRUE	: bit := '1';
constant FALSE	: bit := '0';

-- Width of Data
constant BITWIDTH : integer := 7;  -- If 8 Bits are used BITWIDTH must be 7


-----------------------------------------------------------------------
-- States for Wavelettransform Statemachine
-----------------------------------------------------------------------
type STATE_TYPE is (
			IDLE,
			READ_C0,
			READ_D0,
			READ_C1,
			READ_D1,
			READ_Ci,
			READ_Di,
			FIN_1,
			FIN_2,
			FIN_3,
			FIN_4);

signal currState, nextState		: STATE_TYPE;
signal iInL3,InL1,InL2,InL3		: signed(BITWIDTH downto 0);
signal CalcL1,CalcL2,CalcL3		: signed(BITWIDTH downto 0);
signal X_2i,iX_2i,X_2i_old			: signed(BITWIDTH downto 0);
signal add1,add2						: signed(BITWIDTH+1 downto 0);
signal X_2i_1							: signed(BITWIDTH downto 0);
signal cLatchEn						: bit;
signal copyX_En						: bit;
signal doClkEn							: std_logic;
signal setClkEn						: std_logic;


begin

-- ====================================================================
-- Input Latches
-- ====================================================================
	process (clk,clk_en_in)
	begin
		if clk'event and clk=HI then
			if clk_en_in=HI then
				InL1 <= signed(data_in);
				InL2 <= InL1;
				InL3 <= InL2;
			else
				InL1 <= InL1;
				InL2 <= InL2;
				InL3 <= InL3;
			end if;
		end if;
	end process;

-- Switch for LeftEdge special case
-----------------------------------------------------------------------
	with currState select
		iInL3 <= 	InL1 when READ_C1,
						InL3 when others;

-- Switch for RightEdge special case
-----------------------------------------------------------------------
	with currState select
		iX_2i <= 	X_2i_old when FIN_3,
						X_2i_old when FIN_4,
						X_2i 		when others;


-- CalcLatch Enable
-----------------------------------------------------------------------
	with currState select
		cLatchEn <= 	TRUE when READ_C1,
							TRUE when READ_Ci,
							TRUE when FIN_1,
							TRUE when FIN_3,
							FALSE when others;

-- ====================================================================
-- Latches in front of the arith. for x[2(i)]
-- ====================================================================
	process (clk,cLatchEn,clk_en_in)
	begin
		if clk'event and clk=HI then
			if ((cLatchEn=TRUE) and (clk_en_in=HI)) then
				CalcL1 <= InL1;
				CalcL2 <= InL2;
				CalcL3 <= iInL3;
			else
				CalcL1 <= CalcL1;
				CalcL2 <= CalcL2;
				CalcL3 <= CalcL3;
			end if;
		end if;
	end process;


-- Enable Copy of x[2(i)] to X_2i_old (x[2(i-1)])
-----------------------------------------------------------------------
	with currState select
		copyX_En <= 	TRUE when READ_D1,
							TRUE when READ_Di,
							TRUE when FIN_2,
							FALSE when others;

-- ====================================================================
-- Latch for X_2i_old ( x[2(i-1)] )
-- ====================================================================
	process (clk,copyX_En,clk_en_in)
	begin
		if clk'event and clk=HI then
			if ((copyX_En=TRUE) and (clk_en_in=HI)) then
				X_2i_old <= X_2i;
			else
				X_2i_old <= X_2i_old;
			end if;
		end if;
	end process;


-- ====================================================================
-- Arithmetic for x[2(i)]
-- ====================================================================
	add1 <= (CalcL1(BITWIDTH) & CalcL1) + (CalcL3(BITWIDTH) & CalcL3);
	-- divide by 4 and add
	X_2i <= CalcL2 + (add1(BITWIDTH+1) &  add1(BITWIDTH+1 downto 2));


-- ====================================================================
-- Arithmetic for x[2(i)-1]
-- ====================================================================
	add2 <= (iX_2i(BITWIDTH) & iX_2i) + (X_2i_old(BITWIDTH) & X_2i_old);
	-- divide by 2 and subtract
	X_2i_1 <= (add2(BITWIDTH+1 downto 1)) - CalcL3;


-- ====================================================================
-- clk_en_out generation
-- ====================================================================
	process (clk,doClkEn)
	begin
		if clk'event and clk=HI then
			if doClkEn=YES then
				setClkEn <= clk_en_in;
			else
				setClkEn <= LO;
			end if;
		end if;
	end process;
-- clk_en_out has to be set with negativ clk edge 
-----------------------------------------------------------------------
	process (clk)
	begin
		if clk'event and clk=LO then
			clk_en_out <= setClkEn;
		end if;
	end process;

-- ====================================================================
-- MUX the data_out
-- ====================================================================
	with currState select
		data_out <= 	std_logic_vector(X_2i_1) when READ_Di,
							std_logic_vector(X_2i_1) when FIN_2,
							std_logic_vector(X_2i_1) when FIN_4,
							std_logic_vector(X_2i_old) when others;




-- ====================================================================
-- Implementation of the Statemachine
-- ====================================================================
	-- sequential Part
	process (clk,reset)
	begin
		if reset=HI then
			currState <= IDLE;
		elsif clk'event and clk=HI then
			currState <= nextState;
		end if;
	end process;

	-- comb. Part
	process (currState, reset, enablen_in, clk_en_in)
	begin
		if reset=HI then
			nextState <= IDLE;
			enablen_out <= HI;
			doClkEn	 	<= NO;
		else
			enablen_out <= HI;
			doClkEn	 	<= NO;

			case currState is

		  	-- Waiting for enablen
			when IDLE =>
				if enablen_in = LO then
					nextState <= READ_C0;
				else
					nextState <= IDLE;
				end if;
			
			-- Read the first approx. Coeff.
			when	READ_C0 =>
				if clk_en_in = HI then
					nextState <= READ_D0;
				else
					nextState <= READ_C0;
				end if;

			when READ_D0 =>
				if clk_en_in = HI then
					nextState <= READ_C1;
				else
					nextState <= READ_D0;
				end if;

			when READ_C1 =>
				if clk_en_in = HI then
					nextState <= READ_D1;
				else
					nextState <= READ_C1;
				end if;

			when READ_D1 =>
				doClkEn	 	<= YES;
				enablen_out <= LO;
				if clk_en_in = HI then
					nextState <= READ_Ci;
				else
					nextState <= READ_D1;
				end if;

			when READ_Ci =>
				doClkEn	 	<= YES;
				enablen_out <= LO;
				if clk_en_in = HI then
					nextState <= READ_Di;
				else
					nextState <= READ_Ci;
				end if;

			when READ_Di =>
				doClkEn	 	<= YES;
				enablen_out <= LO;
				if ((enablen_in = HI) and (clk_en_in = HI)) then
					nextState <= FIN_1;
				elsif (clk_en_in = HI) then
					nextState <= READ_Ci;
				else
					nextState <= READ_Di;
				end if;

			when FIN_1 =>
				doClkEn	 	<= YES;
				enablen_out <= LO;
				if clk_en_in = HI then
					nextState <= FIN_2;
				else
					nextState <= FIN_1;
				end if;
				
			when FIN_2 =>
				doClkEn	 	<= YES;
				enablen_out <= LO;
				if clk_en_in = HI then
					nextState <= FIN_3;
				else
					nextState <= FIN_2;
				end if;

			when FIN_3 =>
				doClkEn	 	<= YES;
				enablen_out <= LO;
				if clk_en_in = HI then
					nextState <= FIN_4;
				else
					nextState <= FIN_3;
				end if;

			when FIN_4 =>
				enablen_out <= LO;
				if clk_en_in = HI then
					nextState <= IDLE;
				else
					nextState <= FIN_4;
				end if;

			when others =>
				nextState <= IDLE;

			end case;
		end if;
	end process;

end behavioral;
