library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity receive_frame is
	port ( 	clk 			: in std_logic;
				reset 		: in std_logic;
				h_size		: in std_logic_vector(9 downto 0);
				v_size		: in std_logic_vector(9 downto 0);
				h_offset		: in std_logic_vector(9 downto 0);
				v_offset		: in std_logic_vector(9 downto 0);
				data 			: out std_logic_vector(7 downto 0);
				adr 			: out std_logic_vector(18 downto 0);
				ce 			: out std_logic;
				oe 			: out std_logic;
				we 			: out std_logic;
				VPO 			: out std_logic;
				FSE 			: out std_logic;
				RCV 			: in std_logic;
				VP 			: in std_logic;
				VM 			: in std_logic;
				SUSPND 		: out std_logic;
				OEn 			: out std_logic;
				busy 			: out std_logic
				);
end receive_frame;


-- This entity receives a frame using USB
-- The frame is stored to the external memory
-- Offset and size can be set using h_size,v_size,
-- h_offset and v_offset
--
-- Altough USB is used to transmit data, data trans-
-- mission is not using USB standard
--
-- !! Do not use other USB devices on the same bus !!
-- !! when using this receiver                     !!
--


architecture behavioral of receive_frame is

-- ____________________________________________________________________
-- ====================================================================
-- Entity declarations
-- ====================================================================
component USB_receiver is
	generic(	CLK_DIV: std_logic_vector(5 downto 0) := "011111" );
   Port ( 
	 			reset 		: in std_logic;
				clk 			: in std_logic;								-- should be 50MHz
				RCV			: in 	std_logic;
				VP				: in 	std_logic;
				VM				: in 	std_logic;
				data 			: out std_logic_vector(7 downto 0);
				bitwidth 	: out std_logic_vector(2 downto 0);		-- if 8Bits received bitwidth is 7
				data_ready 	: out std_logic;
				VPO 			: out std_logic;
				FSE 			: out std_logic;
				SUSPND		: out std_logic;
				OEn			: out std_logic);
end component;

component decoder is
    port (
				data_in : in std_logic_vector(7 downto 0);
				bitwidth : in std_logic_vector(2 downto 0);
				data_out : out std_logic_vector(7 downto 0)
			);
end component;


-- ____________________________________________________________________
-- ====================================================================
-- constants declarations
-- ====================================================================


constant HI : std_logic 	:= '1';
constant LO : std_logic 	:= '0';
constant YES : std_logic 	:= '1';
constant NO : std_logic 	:= '0';


type STATE_TYPE is (
			IDLE,
			WAITSTATE1,
			WRITE,
			INCPIX,
			INCLINE,
			WAITING,
			EOF
	);



-- ____________________________________________________________________
-- ====================================================================
-- signal declarations
-- ====================================================================
signal cState,nState 		: STATE_TYPE;
signal data_ready 			: std_logic;
signal bitwidth 				: std_logic_vector(2 downto 0);
signal data_out 				: std_logic_vector(7 downto 0);
signal bitwidth_latched		: std_logic_vector(2 downto 0);
signal data_undec				: std_logic_vector(7 downto 0);
signal pix_cnt					: std_logic_vector(9 downto 0);
signal line_cnt				: std_logic_vector(9 downto 0);
signal pix_adr					: std_logic_vector(9 downto 0);
signal line_adr				: std_logic_vector(9 downto 0);
signal reset_pix 				: std_logic;		-- resets pixel counter
signal inc_pix 				: std_logic;		-- increments pixel counter
signal line_end 				: std_logic;		-- signals end of line
signal reset_line 			: std_logic;		-- resets line counter
signal inc_line 				: std_logic;		-- increments line counter
signal frame_end 				: std_logic;		-- signals end of frame
signal set_writeEn			: std_logic;		-- sets the we signal for ext. RAM



begin
-- ____________________________________________________________________
-- ====================================================================
-- Include Components
-- ====================================================================
rec: USB_receiver 
	generic map (CLK_DIV => "000111")
	port map ( 
	 			reset 		=> reset,
				clk 			=> clk,
				RCV			=> RCV,
				VP				=> VP,
				VM				=> VM,
				data 			=> data_out,
				bitwidth 	=> bitwidth,
				data_ready 	=> data_ready,
				VPO 			=> VPO,
				FSE 			=> FSE,
				SUSPND		=> SUSPND,
				OEn			=> OEn
	);

decode: decoder port map
     (
				data_in 		=> data_undec, -- data_out from USB rec.
				bitwidth 	=> bitwidth_latched,
				data_out 	=> data
		);

-- ____________________________________________________________________
-- ====================================================================
-- RAM Acces signals
-- ====================================================================
	ce <= LO; -- constantly enable RAM
	oe <= HI; -- constantly disable data out
	process(clk)
	begin
		if clk'event and clk=LO then
			we <= not set_writeEn;
		end if;
	end process;

	line_adr <= line_cnt + v_offset;
	pix_adr  <= pix_cnt + h_offset;
	adr <= (line_adr(8 downto 0) & pix_adr(9 downto 0));

	-- Latch data from the usb receiver
	process(clk, data_ready)
	begin
		if clk'event and clk=HI then
			if data_ready = HI then
				data_undec <= data_out;
				bitwidth_latched <= bitwidth;
			end if;
		end if;
	end process;




-- ____________________________________________________________________
-- ====================================================================
-- Counters for line and pixel
-- ====================================================================
	process(clk,reset_pix,inc_pix)
	begin
		if (clk'event and clk=HI) then
			if reset_pix = HI then
				pix_cnt <= (others => '0');
			elsif inc_pix = HI then
				pix_cnt <= pix_cnt + 1;
			end if;
		end if;
	end process;
-----------------------------------------------------------------------
-- Signal for end of line
-----------------------------------------------------------------------
	process(pix_cnt,h_size)
	begin
		if pix_cnt = h_size  then
			line_end <= HI;
		else
			line_end <= LO;
		end if;
	end process;


	process(clk,reset_line,inc_line)
	begin
		if (clk'event and clk=HI) then
			if reset_line = HI then
				line_cnt <= (others => '0');
			elsif inc_line = HI then
				line_cnt <= line_cnt + 1;
			end if;
		end if;
	end process;
-----------------------------------------------------------------------
-- Signal for end of frame
-----------------------------------------------------------------------
	process(line_cnt,v_size)
	begin
		if line_cnt = v_size  then
			frame_end <= HI;
		else
			frame_end <= LO;
		end if;
	end process;


-- ____________________________________________________________________
-- ====================================================================
-- Statemachine
-- ====================================================================
	process(clk,reset)
	begin
		if reset=HI then
			cState <= IDLE;
		elsif clk'event and clk=HI then
			cState <= nState;
		end if;
	end process;


	process(cState, data_ready, frame_end, line_end)
	begin
		busy <= HI;
		inc_pix <= LO;
		set_writeEn <= LO;
		reset_line <= LO;
		inc_line <= LO;
		reset_pix <= LO;
		inc_pix <= LO;

		case cState is

			when IDLE =>
				busy <= LO;
				reset_pix <= HI;
				reset_line <= HI;
				if data_ready=HI then
					nState <= WAITSTATE1;
				else
					nState <= IDLE;
				end if;

			when WAITSTATE1 =>
				nState <= WRITE;
					
			when WRITE =>
				set_writeEn <= HI;
				if ((frame_end=HI) and (line_end=HI)) then
					nState <= EOF;
				elsif ((frame_end=LO) and (line_end=HI)) then
					nState <= INCLINE;
				else
					nState <= INCPIX;
				end if;

			when INCPIX =>
				inc_pix 	<= HI;
				nState 	<= WAITING;
			

			when INCLINE =>
				reset_pix <= HI;
				inc_line <= HI;
				nState 	<= WAITING;

			when WAITING =>
				if data_ready=HI then
					nState <= WRITE;
				else
					nState <= WAITING;
				end if;
			
			when EOF =>
				nState <= IDLE;


			when others =>
				nState <= IDLE;

		end case;
	end process;
			


end behavioral;
