library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;


--
-- Initialize the SAA7113 video decoder chip through
-- the I2C bus.
--
entity saa7113init is
	generic(
		CLK_FREQ: integer := 50_000 -- Virtex clock freq in KHz
	);
	port(
		reset : in std_logic;
		clk : in std_logic;
		scl_in : in std_logic;		-- input from I2C clock signal
		scl_out : out std_logic;	-- output to I2C clock signal
		sda_in : in std_logic;		-- input from I2C data signal
		sda_out : out std_logic		-- output to I2C data signal
	);
end saa7113init;


architecture saa7113init_arch of saa7113init is

constant NO		: std_logic := '0';
constant YES	: std_logic := '1';
constant LO		: std_logic := '0';
constant HI		: std_logic := '1';

--
-- I2C transmitter component with transfer rate of 100 KHz
--
component i2c_tx
	generic(
		NUM_OF_DATABITS	: integer;	-- size of I2C data (usually 8 bits)
		CLK_FREQ 		: integer	-- input clock in KHz
	);
	port(
		reset	: in	std_logic; -- reset the transmitter
		clk		: in	std_logic; -- input clock (divided down internally)
		data_in	: in	std_logic_vector(NUM_OF_DATABITS-1 downto 0); -- data to transmit
		wr		: in	std_logic; -- write control (active-high on rising edge of clk)
		bsy		: out	std_logic; -- transmitter busy
		tx_failed : out	std_logic; -- transmitter failed (NACK or failed arbitration)
		sda_out	: out	std_logic; -- I2C data signal output
		sda_in	: in	std_logic; -- I2C data signal input
		scl_out	: out	std_logic; -- I2C clock signal output
		scl_in	: in	std_logic  -- I2C clock signal input
	);
end component;

-- I2C device address for the SAA7113
constant DEV_ADDR : std_logic_vector(7 downto 0) := "01001010";

-- ROM with initialization data for the SAA7113 registers stored
-- as (register address, register data)
constant ROM_SIZE		: integer := 98;	-- initialize 98/2=49 registers
constant END_ADDRESS	: integer := ROM_SIZE-1;
subtype rom_word is std_logic_vector(7 downto 0);
type rom_table is array (0 to END_ADDRESS) of rom_word;
constant SAA7113_INIT_DATA : rom_table := (
	-- horizontal increment delay; recommended value is 0x08
	rom_word'("00000001"), rom_word'("00001000"), -- (1) <= 8
	-- get composite video from AI22 (RCA jack) with anit-alias filter active
	rom_word'("00000010"), rom_word'("11000011"), -- (2) <= c3
	-- AGC active, white-peak control active
	rom_word'("00000011"), rom_word'("00000000"), -- (3) <= 0
	-- horizontal sync beginning
	rom_word'("00000110"), rom_word'("11101001"), -- (6) <= e9
	-- horizontal sync stop
	rom_word'("00000111"), rom_word'("00001101"), -- (7) <= d
	-- normal vertical noise reduction; horiz. PLL closed; fast horiz. locking; auto field detection
	rom_word'("00001000"), rom_word'("10011000"), -- (8) <= 98
	-- aperture factor=0; AGC update once-per-frame; active lumiance bypass; 4.1 MHz aperture center; prefilter bypassed; chromiance trap bypassed
	rom_word'("00001001"), rom_word'("00000100"), -- (9) <= 4
	-- luminance brightness control
	rom_word'("00001010"), rom_word'("10000000"), -- (a) <= 80
	-- luminance contrast control
	rom_word'("00001011"), rom_word'("01000111"), -- (b) <= 47
	-- chromiance saturation
	rom_word'("00001100"), rom_word'("01000000"), -- (c) <= 40
	-- chromiance hue
	rom_word'("00001101"), rom_word'("00000000"), -- (d) <= 0
	-- chromiance bandwidth=800 KHz; nominal color time constant; chromiance comb filter on; NTSC color standard
	rom_word'("00001110"), rom_word'("00100010"), -- (e) <= 22
	-- nominal chromiance gain
	rom_word'("00001111"), rom_word'("00100100"), -- (f) <= 24
	-- 
	rom_word'("00010000"), rom_word'("00000000"), -- (10) <= 0
	-- color forced on; processed data to VPO; RTS0/1, RTC0 active; VPO bus active
	rom_word'("00010001"), rom_word'("00001101"), -- (11) <= d
	-- RTS0/1 control
	rom_word'("00010010"), rom_word'("10100111"), -- (12) <= a7
	-- output control
	rom_word'("00010011"), rom_word'("00000000"), -- (13) <= 0
	-- start of vgate pulse
	rom_word'("00010101"), rom_word'("00000011"), -- (15) <= 3
	-- stop of vgate pulse
	rom_word'("00010110"), rom_word'("00000101"), -- (16) <= 5
	-- vgate start/stop MSBs
	rom_word'("00010111"), rom_word'("00000011"), -- (17) <= 3
	-- data slicer clock selection; amplitude searching; framing code error; Hamming check; field size select
	rom_word'("01000000"), rom_word'("11100010"), -- (40) <= e2
	-- video format - active video
	rom_word'("01000001"), rom_word'("11111111"), -- (41) <= ff
	rom_word'("01000010"), rom_word'("11111111"), -- (42) <= ff
	rom_word'("01000011"), rom_word'("11111111"), -- (43) <= ff
	rom_word'("01000100"), rom_word'("11111111"), -- (44) <= ff
	rom_word'("01000101"), rom_word'("11111111"), -- (45) <= ff
	rom_word'("01000110"), rom_word'("11111111"), -- (46) <= ff
	rom_word'("01000111"), rom_word'("11111111"), -- (47) <= ff
	rom_word'("01001000"), rom_word'("11111111"), -- (48) <= ff
	rom_word'("01001001"), rom_word'("11111111"), -- (49) <= ff
	rom_word'("01001010"), rom_word'("11111111"), -- (4a) <= ff
	rom_word'("01001011"), rom_word'("11111111"), -- (4b) <= ff
	rom_word'("01001100"), rom_word'("11111111"), -- (4c) <= ff
	rom_word'("01001101"), rom_word'("11111111"), -- (4d) <= ff
	rom_word'("01001110"), rom_word'("11111111"), -- (4e) <= ff
	rom_word'("01001111"), rom_word'("11111111"), -- (4f) <= ff
	rom_word'("01010000"), rom_word'("11111111"), -- (50) <= ff
	rom_word'("01010001"), rom_word'("11111111"), -- (51) <= ff
	rom_word'("01010010"), rom_word'("11111111"), -- (52) <= ff
	rom_word'("01010011"), rom_word'("11111111"), -- (53) <= ff
	rom_word'("01010100"), rom_word'("11111111"), -- (54) <= ff
	rom_word'("01010101"), rom_word'("11111111"), -- (55) <= ff
	rom_word'("01010110"), rom_word'("11111111"), -- (56) <= ff
	rom_word'("01010111"), rom_word'("11111111"), -- (57) <= ff
	-- programmable framing code
	rom_word'("01011000"), rom_word'("01000000"), -- (58) <= 40
	-- horizontal offset
	rom_word'("01011001"), rom_word'("01010100"), -- (59) <= 54
	-- vertical offset
	rom_word'("01011010"), rom_word'("00001010"), -- (5a) <= a
	-- MSBs for vertical and horizontal offsets
	rom_word'("01011011"), rom_word'("00010011"), -- (5b) <= 13
	-- SDID codes
	rom_word'("01011110"), rom_word'("00000000")  -- (5e) <= 0
);

-- state definitions for initialization state machine
type init_state is (
	init,			-- initialize the I2C tx and ROM table address counter
	wr_dev_addr,	-- address the SAA7113 chip
	wr_dev_reg1,	-- now send the SAA7113 register address
	wr_dev_reg2,	-- wait for the I2C transmitter to accept the register address
	wr_dev_data1,	-- now send the data to store in the SAA7113 register
	wr_dev_data2,	-- wait for the I2C transmitter to accept the register data
	check_end,		-- check to see if all registers are initialized
	done
);

signal init_next	: init_state;	-- state of initialization state machine
signal i2c_reset	: std_logic;	-- reset input to I2C transmitter
signal data_in		: std_logic_vector(7 downto 0);	-- data input to I2C transmitter
signal wr			: std_logic;	-- write control to I2C transmitter
signal bsy			: std_logic;	-- I2C transmitter busy status signal
signal tx_failed	: std_logic;	-- I2C transmitter failed status signal

subtype addr_counter is integer range 0 to END_ADDRESS;
signal addr			: addr_counter;	-- address into the ROM with initialization data

begin

	-- instantiate the I2C transmitter component
	u0: i2c_tx
		generic map(
			NUM_OF_DATABITS => 8,
			CLK_FREQ => CLK_FREQ
		)
		port map(
			reset => i2c_reset,
			clk => clk,
			data_in => data_in,
			wr => wr,
			bsy => bsy,
			tx_failed => tx_failed,
			sda_out => sda_out,
			sda_in => sda_in,
			scl_out => scl_out,
			scl_in => scl_in
		);	

	-- state machine that loads the SAA7113 using the I2C transmitter
	-- and then halts
	process(reset,clk)
	begin

		if(reset=YES) then
			init_next <= init;	-- go to initial state on reset

		elsif(clk'event and clk=HI) then

			-- default values for some I2C transmitter control signals
			i2c_reset <= NO;
			wr <= NO;

			case init_next is

			when init =>	-- initialize the I2C tx and ROM table address counter
				i2c_reset <= YES;
				addr <= 0;
				if(bsy=NO and tx_failed=NO) then
					-- only move to the next state when the I2C transmitter
					-- is not busy and not reporting failed transmissions
					init_next <= wr_dev_addr;
				end if;

			when wr_dev_addr => -- address the SAA7113 chip
				data_in <= DEV_ADDR;	-- send SAA7113 device address to I2C transmitter
				wr <= YES;				-- write the device address into the I2C tx
				if(tx_failed=YES) then
					init_next <= init;	-- redo entire SAA7113 init if any transmission fails
				elsif(bsy=YES) then
					-- done latching address into I2C tx once it signals it is busy
					-- so go to the next state
					init_next <= wr_dev_reg1;
				end if;

			when wr_dev_reg1 =>	-- now send the SAA7113 register address
				data_in <= SAA7113_INIT_DATA(addr);	-- send register address from ROM to I2C tx
				wr <= YES;				-- write the register address into the I2C tx
				if(tx_failed=YES) then
					init_next <= init;	-- redo entire SAA7113 init if any transmission fails
				elsif(bsy=NO) then
					-- previous device address transmission is not done until I2C tx
					-- is not busy.  So wait in this state for tx not busy.
					init_next <= wr_dev_reg2;
				end if;

			when wr_dev_reg2 =>	-- wait for the I2C transmitter to accept the register address
				data_in <= SAA7113_INIT_DATA(addr);	-- send register address from ROM to I2C tx
				wr <= YES;				-- write the register address into the I2C tx
				if(tx_failed=YES) then
					init_next <= init;	-- redo entire SAA7113 init if any transmission fails
				elsif(bsy=YES) then
					-- the I2C transmitter is busy again, which means it has
					-- accepted the register address.
					addr <= addr + 1;	-- now address the data for the register
					init_next <= wr_dev_data1;
				end if;

			when wr_dev_data1 => -- now send the data to store in the SAA7113 register
				data_in <= SAA7113_INIT_DATA(addr);	-- send register data from ROM to I2C tx
				wr <= YES;				-- write the register data into the I2C tx
				if(tx_failed=YES) then
					init_next <= init;	-- redo entire SAA7113 init if any transmission fails
				elsif(bsy=NO) then
					-- previous register address transmission is not done until I2C tx
					-- is not busy.  So wait in this state for tx not busy.
					init_next <= wr_dev_data2;
				end if;

			when wr_dev_data2 => -- wait for the I2C transmitter to accept the register data
				data_in <= SAA7113_INIT_DATA(addr);	-- send register data from ROM to I2C tx
				wr <= YES;				-- write the register data into the I2C tx
				if(tx_failed=YES) then
					init_next <= init;	-- redo entire SAA7113 init if any transmission fails
				elsif(bsy=YES) then
					-- the I2C transmitter is busy again, which means it has
					-- accepted the register address.
					addr <= addr + 1;	-- now address the next register address
					init_next <= check_end;
				end if;

			when check_end => -- check to see if all registers are initialized
				data_in <= SAA7113_INIT_DATA(addr);
				wr <= NO;
				if(tx_failed=YES) then
					init_next <= init;	-- redo entire SAA7113 init if any transmission fails
				elsif(addr=END_ADDRESS) then
					-- done if this is the last table entry
					init_next <= done;
				elsif(bsy=NO) then
					-- otherwise, do next entry in initialization table
					init_next <= wr_dev_addr;
				end if;

			when done => -- SAA7113 initialization done so wait here
				init_next <= done;

			when others => -- send erroneous states back to initial state
				init_next <= init;

			end case;

		end if;

	end process;

end saa7113init_arch;






