library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity transmit_frame is
	port ( 
				reset 		: in std_logic;
				clk 			: in std_logic;
				trigger 		: in std_logic;
				h_size		: in std_logic_vector(9 downto 0);
				v_size		: in std_logic_vector(9 downto 0);
				h_offset		: in std_logic_vector(9 downto 0);
				v_offset		: in std_logic_vector(9 downto 0);
				data 			: in std_logic_vector(7 downto 0);
				adr 			: out std_logic_vector(18 downto 0);
				ce 			: out std_logic;
				oe 			: out std_logic;
				we 			: out std_logic;
				VPO 			: out std_logic;
				FSE 			: out std_logic;
				SUSPND 		: out std_logic;
				OEn 			: out std_logic;
				busy 			: out std_logic);
end transmit_frame;

-- This entity transmits a frame using USB
-- The frame is read from the external memory
-- Offset and size can be set using h_size,v_size,
-- h_offset and v_offset
--
-- Altough USB is used to transmit data, data trans-
-- mission is not using USB standard
--
-- !! Do not use other USB devices on the same bus !!
-- !! when using this transmitter                  !!
--


architecture behavioral of transmit_frame is

-- ____________________________________________________________________
-- ====================================================================
-- Entity declarations
-- ====================================================================
component USB_transmitter is
	generic(	CLK_DIV: std_logic_vector(5 downto 0) := "011111" );
   Port ( 
	 			reset 		: in std_logic;
				clk 			: in std_logic;								-- should be 50MHz
				data 			: in std_logic_vector(7 downto 0);
				bitwidth 	: in std_logic_vector(2 downto 0);		-- if sending 8Bits bitwidth is 7
				send_data 	: in std_logic;
				VPO 			: out std_logic;
				FSE 			: out std_logic;
				SUSPND		: out std_logic;
				OEn			: out std_logic;
				busy 			: out std_logic);
end component;


component coder is
    port (
				data_in 		: in std_logic_vector(7 downto 0);
				data_out 	: out std_logic_vector(7 downto 0);
				bitwidth 	: out std_logic_vector(2 downto 0)
			);
end component;



-- ____________________________________________________________________
-- ====================================================================
-- constants declarations
-- ====================================================================

constant bitwidth_fix : std_logic_vector(2 downto 0) := "111";

constant HI : std_logic 	:= '1';
constant LO : std_logic 	:= '0';
constant YES : std_logic 	:= '1';
constant NO : std_logic 	:= '0';


type STATE_TYPE is (
			IDLE,
			WAITSTATE1,
			WAITSTATE2,
			WAITING,
			SET_SEND,
			INCLINE,
			EOF
	);

-- ____________________________________________________________________
-- ====================================================================
-- signal declarations
-- ====================================================================
signal cState,nState :	STATE_TYPE;
signal send_data : 	std_logic;		-- triggers the transmitter to send data
signal set_send_data : 	std_logic;		-- triggers the transmitter to send data
signal busy_tr : 		std_logic;		-- HI during transmission of data
signal pix_cnt:		std_logic_vector(9 downto 0);
signal line_cnt:		std_logic_vector(9 downto 0);
signal pix_adr:		std_logic_vector(9 downto 0);
signal line_adr:		std_logic_vector(9 downto 0);
signal reset_pix : 	std_logic;		-- resets pixel counter
signal inc_pix : 		std_logic;		-- increments pixel counter
signal line_end : 	std_logic;		-- signals end of line
signal reset_line : 	std_logic;		-- resets line counter
signal inc_line : 	std_logic;		-- increments line counter
signal frame_end : 	std_logic;		-- signals end of frame
signal data_coded:	std_logic_vector(7 downto 0);
signal bitwidth:		std_logic_vector(2 downto 0);

signal dummy1 : 	std_logic;		-- signals end of frame



begin

-- ____________________________________________________________________
-- ====================================================================
-- Include Components
-- ====================================================================
trans: USB_transmitter 
	generic map (CLK_DIV => "000111")	-- 7 => f_trans=6.25MHz
	port map ( 	 			
				reset 		=> reset,
				clk 			=> clk,
				data 			=> data_coded,
				bitwidth 	=> bitwidth,
				send_data 	=> send_data,
				VPO 			=> VPO,
				FSE 			=> FSE,
				SUSPND		=> SUSPND,
				OEn			=> OEn,
				busy 			=> busy_tr
	);

code: coder port map (
				data_in 		=> data,
				data_out 	=> data_coded,
				bitwidth 	=> bitwidth
	);




-- ____________________________________________________________________
-- ====================================================================
-- RAM Acces signals
-- ====================================================================
	ce <= LO; -- constantly enable RAM
	oe <= LO; -- constantly enable data out
	we <= HI; -- constantly disable write

	line_adr <= line_cnt + v_offset;
	pix_adr  <= pix_cnt + h_offset;
	adr <= (line_adr(8 downto 0) & pix_adr(9 downto 0));


-- ____________________________________________________________________
-- ====================================================================
-- Counters for Pixel and Line
-- ====================================================================
	process(clk,reset_pix,inc_pix)
	begin
		if (clk'event and clk=HI) then
			if reset_pix = HI then
				pix_cnt <= (others => '0');
			elsif inc_pix = HI then
				pix_cnt <= pix_cnt + 1;
			end if;
		end if;
	end process;
-----------------------------------------------------------------------
-- Signal for end of line
-----------------------------------------------------------------------
	process(pix_cnt,h_size)
	begin
		if pix_cnt = h_size then
			line_end <= HI;
		else
			line_end <= LO;
		end if;
	end process;


	process(clk,reset_line,inc_line)
	begin
		if (clk'event and clk=HI) then
			if reset_line = HI then
				line_cnt <= (others => '0');
			elsif inc_line = HI then
				line_cnt <= line_cnt + 1;
			end if;
		end if;
	end process;
-----------------------------------------------------------------------
-- Signal for end of frame
-----------------------------------------------------------------------
	process(line_cnt,v_size)
	begin
		if line_cnt = v_size then
			frame_end <= HI;
		else
			frame_end <= LO;
		end if;
	end process;


-- ____________________________________________________________________
-- ====================================================================
-- send_data signal
-- ====================================================================
	process(clk)
	begin
		if	clk'event and clk=LO then
			send_data <= set_send_data;
		end if;
	end process;

-- ____________________________________________________________________
-- ====================================================================
-- Statemachine
-- ====================================================================
	process(clk,reset)
	begin
		if reset=HI then
			cState <= IDLE;
		elsif clk'event and clk=HI then
			cState <= nState;
		end if;
	end process;


	process(cState, trigger, busy_tr, frame_end, line_end)
	begin
		busy <= HI;
		inc_pix <= LO;
		set_send_data <= LO;
		reset_line <= LO;
		inc_line <= LO;
		reset_pix <= LO;
		inc_pix <= LO;

		case cState is

			when IDLE =>
				busy <= LO;
				reset_line <= HI;
				reset_pix  <= HI;
				if trigger = HI then
					nState <= WAITSTATE1;
				else
					nState <= IDLE;
				end if;

			when WAITSTATE1 =>
					nState <= WAITSTATE2;
			when WAITSTATE2 =>
					nState <= WAITING;


			when WAITING =>
				if busy_tr = LO then
					nState <= SET_SEND;
				else
					nState <= WAITING;
				end if;
				
			when SET_SEND =>
				inc_pix <= HI;
				set_send_data <= HI;
				if ((frame_end = HI) and (line_end=HI)) then
					nState <= EOF;
				elsif ((frame_end = LO) and (line_end=HI)) then
					nState <= INCLINE;
				else
					nState <= WAITING;
				end if;

		  	when INCLINE =>
				inc_line <= HI;
				reset_pix <= HI;
				nState <= WAITING;

			when EOF =>
				nState <= IDLE;

			when others =>
				nState <= IDLE;

		end case;
	end process;



end behavioral;
