library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity wt_1Up_8 is
    Port ( resetn : 		in std_logic;
           clk : 			in std_logic;
           trigger : 	in std_logic;
			  horOffset : 	in	 std_logic_vector(9 downto 0); 	--Picture Offset in RAM
			  verOffset : 	in	 std_logic_vector(9 downto 0); 	--Picture Offset in RAM
			  h_size:		in  std_logic_vector(9 downto 0);	-- horizontal Picture Size (>16)
			  v_size:		in  std_logic_vector(9 downto 0);	-- vertical Picture Size
           pix_adr: 		out std_logic_vector(9 downto 0);
			  line_adr:		out std_logic_vector(9 downto 0);
           data_in : 	in  std_logic_vector(7 downto 0);
           data_out : 	out std_logic_vector(7 downto 0);
			  write_data : out std_logic;
			  cen : 			out std_logic;		-- Chip Enable
			  oen : 			out std_logic;		-- Output Enable
			  wen : 			out std_logic;		-- Write Enable
           busy : 		out std_logic);
end wt_1Up_8;

-- ====================================================================
-- Design for Inverse Wavelet Transformation on XSV300 Board
-- ====================================================================
--
-- This core reads a frame from the left RAM Bank and sends it
-- to the IWT arithmetic
--
-- USE Xilinx XST Technology to Synthesize; FPGA Express does not
-- generate apropriate results
--
-- 
-- The Arithmetic is set to 8 Bit
--
-- Vers. 1.0  27.09.01 D.Bachofen
-- ====================================================================

architecture behavioral of wt_1Up_8 is

-- ====================================================================
-- Architecture declarations
-- ====================================================================


-- wavelettransformation Core
-----------------------------------------------------------------------
-----------------------------------------------------------------------
-- Components:
-----------------------------------------------------------------------
component cd53_up_8Bit is
	Port ( 	reset 			: in std_logic;	-- active HI reset
				clk 				: in std_logic;
				enablen_in 		: in std_logic;	-- active LO enable
				clk_en_in 		: in std_logic;
				data_in 			: in std_logic_vector(7 downto 0);
				data_out 		: out std_logic_vector(7 downto 0);
				clk_en_out 		: out std_logic;
				enablen_out 	: out std_logic);	-- active LO enable out
end component;

component ramBlock1024k 
    Port ( clk 	: in std_logic;
           we 		: in std_logic;
           adr 	: in std_logic_vector(9 downto 0);
           di 		: in std_logic_vector(7 downto 0);
           do 		: out std_logic_vector(7 downto 0));
end component;


-----------------------------------------------------------------------
-- Constants:
-----------------------------------------------------------------------
constant HI : std_logic := '1';
constant LO : std_logic := '0';
constant YES : std_logic := '1';
constant NO : std_logic := '0';

-- Width of Data
constant BITWIDTH : integer := 7;  -- If 8 Bits are used BITWIDTH must be 7


-----------------------------------------------------------------------
-- States for Wavelettransform Statemachine
-----------------------------------------------------------------------
type STATE_TYPE_READ is (
			IDLE,					
		 	READ_LINE,
		 	RD_C,
		 	RD_D,
			GEN_CLKEN,
			RESET_COUNTER,
		 	WRITE_LINE,	
			WR1,		
			WR2,			
		 	INC_LINE);
	

signal cS_Read, nS_Read: STATE_TYPE_READ; -- states for main statemachine
signal we_Ram : std_logic;	-- write Enable to RAM block (active high)
signal adr_Ram : std_logic_vector(9 downto 0);
signal adr_Ram2 : std_logic_vector(9 downto 0); -- Muxed from pix Counters
signal di_Ram : std_logic_vector(BITWIDTH downto 0);
signal do_RAM : std_logic_vector(BITWIDTH downto 0);
signal reset_pix_cnt : std_logic;
signal d0Pix:  std_logic_vector(9 downto 0);	-- Pixel number detailcoef L0
signal c0Pix:  std_logic_vector(9 downto 0);	-- Pixel number appr.coef L2
signal rPix:  std_logic_vector(9 downto 0);	-- Pixel number that will be read
signal pix_adr_L:  std_logic_vector(9 downto 0);	-- Pixel number that will be read
signal pix_adr_mux:  std_logic_vector(9 downto 0);	-- Pixel number that will be read
signal incPix : std_logic; -- If Hi the pixel counter will be incremented
signal incPixC : std_logic; -- If Hi the pixel counter for C will be incremented
signal readC : std_logic; -- If Hi the pixel counter for C will be incremented
signal incPixD : std_logic; -- If Hi the pixel counter for D will be incremented
signal Line: std_logic_vector(9 downto 0);	-- Line number that will be read
signal readFinished : std_logic; -- YES when entire Line is read
signal clk_en : std_logic; -- Enable for clk wvt Level 0
signal clk_en1 : std_logic;	-- Used to delay clk_en
signal clk_en2 : std_logic;	-- Used to delay clk_en
signal writeFinished: std_logic; -- YES when entire Line is written
signal pictureFinished: std_logic; -- YES when entire Picture is transformed
signal write_1, write_2 : std_logic;	-- signals to write data to memory


signal reset: std_logic; 	-- is /resetn
-- Level 0 transformation
signal enable_L0: std_logic;	-- Enable for Level 0 wvt
signal dout_L0: std_logic_vector(7 downto 0); -- detail coeff from L0
signal dout_L0_L: std_logic_vector(7 downto 0); -- detail coeff from L0
signal clkEn_L0:	std_logic;	-- clk Enable for Level 1
signal en_out_L0:	std_logic; -- Enable for Level 1
signal data_out2wvt: std_logic_vector(7 downto 0);	--Data from RAM to wvt Level0





begin

-- ====================================================================
-- Architecture body
-- ====================================================================

-- wavelettransformation Cores
-----------------------------------------------------------------------
wvt_L0up: cd53_up_8Bit port map (
				reset 			=> reset,
				clk 				=> clk,
				enablen_in 		=> enable_L0,
				clk_en_in 		=> clk_en,
				data_in 			=> data_out2wvt,
				data_out 		=> di_Ram,
				clk_en_out 		=> clkEn_L0,
				enablen_out 	=> en_out_L0
			);


ram1: ramBlock1024k port map (
		clk 		=> clk,
      we 		=> clkEn_L0,
      adr 		=> rPix,
      di 		=> di_Ram,
      do 		=> do_Ram
	);



-- Counter for internal RAM
-----------------------------------------------------------------------
	process (clk, incPix, clkEn_L0)
	begin
   	if (reset_pix_cnt=HI) then
			rPix <= (others => '0');
		elsif (clk'event and clk=HI) then
			if ((incPix=HI) or (clkEn_L0=HI)) then
			-- Increment pixelcounter after read has been performed
				rPix <= rPix + 1;
			end if;
	  	end if;
	end process;




-- ====================================================================
-- Statemachine for read and write to external RAM
-- ====================================================================
	process (clk, resetn)
	begin
		if resetn = LO then
			cS_Read <= IDLE;
		elsif clk'event and clk = HI then
			cS_Read <= nS_Read;
		end if;
	end process;

	process ( cS_Read,trigger,readFinished,
				writeFinished,pictureFinished,
				readC,en_out_L0 )
	begin
		busy <= HI;
		reset_pix_cnt <= LO;
		write_1 <= LO;
		write_2 <= LO;
		clk_en1 <= LO;
		incPix <= LO;
		incPixC <= LO;
		incPixD <= LO;

		case cS_Read is

			when IDLE =>
				busy <= LO;
				reset_pix_cnt <= HI;
				if trigger=HI then
					nS_Read <= READ_LINE;
				else 
					nS_Read <= IDLE;
				end if;

			when READ_LINE =>
				if (readFinished=NO) and (readC=HI) then
					nS_Read <= RD_C;
				elsif (readFinished=NO) and (readC=LO) then
					nS_Read <= RD_D;
				elsif (writeFinished=NO) and  (en_out_L0 = LO) then
					nS_Read <= GEN_CLKEN;
				elsif ((readFinished=YES) and (writeFinished=YES) and  (en_out_L0 = HI)) then
					nS_Read <= RESET_COUNTER;
				else
					nS_Read <= READ_LINE;
				end if;

			when RD_C =>
				incPixC <= HI;
				clk_en1 <= HI;
				nS_Read <= READ_LINE;

			when RD_D =>
				incPixD <= HI;
				clk_en1 <= HI;
				nS_Read <= READ_LINE;

			when GEN_CLKEN =>
				clk_en1 <= HI;
				nS_Read <= READ_LINE;

			when RESET_COUNTER =>
				reset_pix_cnt <= HI;
				nS_Read <= WRITE_LINE;

			when WRITE_LINE =>
				if (writeFinished=YES) then -- writeFinished checks the ext. RAM counter
					nS_Read <= INC_LINE;
				else
					nS_Read <= WR1;
				end if;

			when WR1 =>
				write_1 <= HI;
				nS_Read <= WR2;
										
			when WR2 =>
				incPix <= HI;
				write_2 <= HI;
				nS_Read <= WRITE_LINE;

			when INC_LINE =>
				reset_pix_cnt <= HI;
				if (pictureFinished = HI) then
					nS_Read <= IDLE;
				else 
					nS_Read <= READ_LINE;
				end if;

			when others =>
				nS_Read <= IDLE;				

		end case;
	end process;


-- clk_en generation
-----------------------------------------------------------------------
	process (clk,resetn,incPixC)
	begin
		if resetn = LO then
			readC <= HI;
		elsif clk'event and clk=HI then
			readC <= not incPixC;
		end if;
  	end process;

-- clk_en generation
-----------------------------------------------------------------------
	process (clk)
	begin
		if clk'event and clk=HI then
			clk_en2 <= clk_en1; 
		end if;
   end process;
	process (clk)
	begin
		if clk'event and clk=LO then
			clk_en <= clk_en2; 
		end if;
   end process;

-- enable for WVT
-----------------------------------------------------------------------
	enable_L0 <= readFinished;			


-- Counter for Lines
-----------------------------------------------------------------------
	process (clk, resetn, cS_Read)
	begin
   	if (resetn=LO) then
			Line <= (others => '0');
		elsif (clk'event and clk=HI) then
			if cS_Read=IDLE then 
			-- Linecounter reset after picture is Transformed
				Line <= (others => '0');

			elsif cS_Read=INC_LINE then
			-- Increment lincounter in inc_line State
				Line <= Line + 1;
			end if;
	  	end if;
	end process;



-- Counter for detail coeff L0
-----------------------------------------------------------------------
	process (clk, reset_pix_cnt, incPixD)
	begin
   	if (reset_pix_cnt=HI) then
			d0Pix <= ("0" & h_size(9 downto 1));	-- h_size / 2
		elsif (clk'event and clk=HI) then
			if (incPixD=HI) then
				-- Increment pixelcounter after write has been performed
				d0Pix <= d0Pix + 1;
			end if;
	  	end if;
	end process;


-- Counter for approximation coeff L0
-----------------------------------------------------------------------
	process (clk, reset_pix_cnt, incPixC)
	begin
   	if (reset_pix_cnt=HI) then
			c0Pix <= (others => '0');
		elsif (clk'event and clk=HI) then
			if (incPixC=HI) then
				-- Increment pixelcounter after write has been performed
				c0Pix <= c0Pix + 1;
			end if;
	  	end if;
	end process;





-- ====================================================================
-- MUX and Latch for RAM adress
-- ====================================================================
	with nS_Read select	-- RAM output enable only for read and before read
		pix_adr_mux <= 	d0Pix when RD_D,
								c0Pix when RD_C,
								rPix when others;
	process (clk)
	begin
		if clk'event and clk=HI then
			pix_adr_L <= pix_adr_mux;
		end if;
	end process;

	

	pix_adr <= horOffset	+ pix_adr_L;

	line_adr <= Line + verOffset;


-- ====================================================================
-- RAM Acces Signals
-- ====================================================================
	cen <= LO;	-- RAM always enabled

	with cS_Read select	-- RAM output enable only for read and before read
		oen <= 	'0' when RD_D,
					'0' when RD_C,
					'1' when others;

	process(clk, write_1,write_2)
	begin
		if clk'event and clk=LO then
			wen <= not write_1;
		end if;
	end process;
			
-- data from RAM to wvt Level 0
-----------------------------------------------------------------------
	process(clk, cS_Read)
	begin
		if clk'event and clk = HI then
			if (cS_Read = RD_D) or (cS_Read = RD_C) then
			 	data_out2wvt <= data_in;
			else
				data_out2wvt <= data_out2wvt;
			end if;
		end if;
	end process;



-- tell the others to read data (from wvt to ext. RAM)
-----------------------------------------------------------------------
	with cS_Read select
			write_data <= 	HI when RESET_COUNTER,
								HI when WRITE_LINE,
								HI when WR1,
								HI when WR2,
								LO when others;



-- data from internal RAM to external RAM
-----------------------------------------------------------------------
	data_out	<= do_RAM;




-- read Finished Signal
-----------------------------------------------------------------------
	process (d0Pix)
	begin
		if d0Pix >= h_size then
			readFinished <= YES;
		else
			readFinished <= NO;
		end if;
	end process;

-- Picture Finished Signal
-----------------------------------------------------------------------
	process (Line)
	begin
		if (Line >= v_size) then
			pictureFinished <= YES;
		else
			pictureFinished <= NO;
		end if;
	end process;

-- Write Finished Signal
-----------------------------------------------------------------------
	process (rPix,en_out_L0)
	begin
		if (rPix >= h_size)  then
			writeFinished <= YES;
		else
			writeFinished <= NO;
		end if;
	end process;



-- ====================================================================
-- some more assignements
-- ====================================================================

	reset <= not resetn;

end behavioral;
