library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity wvt_usb1 is
    Port ( 
	 	clk : in std_logic;								-- program. clock
		lp_d: in std_logic_vector(2 downto 0);		-- parallel port data pin
		lp_s: out std_logic;								-- parallel port status pin
		dipsw: in std_logic_vector (2 downto 1);
		sw1 :	in std_logic;								-- switches
		sw2 :	in std_logic;								-- switches
		sw4 :	in std_logic;								-- switches
		usbvm :	in std_logic;							-- usb VM
		usbvp :	in std_logic;							-- usb VP
		usbrcv :	in std_logic;							-- usb RCV
		usboeb :	out std_logic;							-- usb OEn
		usbvpo :	out std_logic;							-- usb VPO
		usbvmo :	out std_logic;							-- usb VMO/FSEO
		usbsuspnd :	out std_logic;						-- usb SUSPND
		trste : out std_logic;							-- Tristate Ethernet
		llck : in std_logic;								-- Clock from SAA7113
		rts : in std_logic_vector(1 downto 0);		-- Sync Signals from SAA7113
		scl : inout std_logic;							-- bidirectional I2C clock signal
		sda : inout std_logic;							-- bidirectional I2C data signal
		vpo : in std_logic_vector(7 downto 0);		-- videodata from SAA7113
		rs : out std_logic_vector(2 downto 0);		-- RAMDAC register address bus
		d : out std_logic_vector(7 downto 0);		-- Data to RAMDAC Register
		wrb, rdb: out std_logic;						-- write/read strobes to BT481 RAMDAC
		hsyncb: out std_logic;							-- horizontal sync RAMDAC
		vsyncb: out std_logic;							-- vertical sync RAMDAC
		blankb: out std_logic;							-- video blanking RAMDAC
		pixelclk: out std_logic;						-- pixel clock to RAMDAC
		p: out std_logic_vector(7 downto 0);		-- pixeldata bus to RAMDAC
		lceb, loeb, lweb: out std_logic;				-- SRAM control lines
		la: out std_logic_vector(18 downto 0);		-- SRAM address bus
		ld: inout std_logic_vector(15 downto 0);	-- SRAM data bus
		rceb: out std_logic;								-- SRAM control lines
		rd: out std_logic_vector(11 downto 8)
	 );
end wvt_usb1;

-- ====================================================================
-- TopLevel Design for Wavelet Transformation on XSV300 Board
-- including USB based communication with a second XSV 300 Board
-- ====================================================================
--
-- This project reads a picture from a CVBS video cam and stores
-- it to the left RAM bank
-- Then it transforms the frame with a wavelet 5-3 transform
-- You can send the data over USB to a second XSV Board. There is 
-- a Huffman coder to compress data before sending
--
-- The design is controled by the pushbuttons:
-- SW1 = get frame
-- SW2 = WT
-- SW3 = ---
-- SW4 = send data
-- SW1 & SW2 = reset
--
-- USE Xilinx XST Technology to Synthesize; FPGA Express does not
-- generate apropriate results
--
-- 
-- The Arithmetic is set to 8 Bit
--
-- Vers. 1.0  27.09.01 D.Bachofen
-- ====================================================================


architecture behavioral of wvt_usb1 is

-- Picture size and Offset for Transformation cores
constant h_size		: std_logic_vector(9 downto 0) := "1010000000";		--640
constant v_size		: std_logic_vector(9 downto 0) := "0111100000";		--480
constant verOffset	: std_logic_vector(9 downto 0) := "0000011111";		--31
constant horOffset	: std_logic_vector(9 downto 0) := "0000010101";		--20


constant NO  : std_logic := '0';
constant YES : std_logic := '1';
constant LO  : std_logic := '0';
constant HI  : std_logic := '1';

-- Components
--*****************************************************************

-- Component for transmitting frame over usb
component transmit_frame is
	port ( 
				reset 		: in std_logic;
				clk 			: in std_logic;
				trigger 		: in std_logic;
				h_size		: in std_logic_vector(9 downto 0);
				v_size		: in std_logic_vector(9 downto 0);
				h_offset		: in std_logic_vector(9 downto 0);
				v_offset		: in std_logic_vector(9 downto 0);
				data 			: in std_logic_vector(7 downto 0);
				adr 			: out std_logic_vector(18 downto 0);
				ce 			: out std_logic;
				oe 			: out std_logic;
				we 			: out std_logic;
				VPO 			: out std_logic;
				FSE 			: out std_logic;
				SUSPND 		: out std_logic;
				OEn 			: out std_logic;
				busy 			: out std_logic);
end component;

-- component for wavelettransformation
component wt_1Down_8
    Port ( resetn : 		in std_logic;
           clk : 			in std_logic;
           trigger : 	in std_logic;
			  horOffset : 	in	 std_logic_vector(9 downto 0); 	--Picture Offset in RAM
			  verOffset : 	in	 std_logic_vector(9 downto 0); 	--Picture Offset in RAM
			  h_size:		in std_logic_vector(9 downto 0);	-- horizontal Picture Size (>16)
			  v_size:		in std_logic_vector(9 downto 0);	-- vertical Picture Size
           pix_adr: 		out std_logic_vector(9 downto 0);
			  line_adr:		out std_logic_vector(9 downto 0);
           data_in : 	in  std_logic_vector(7 downto 0);
           data_out : 	out std_logic_vector(7 downto 0);
			  write_data : out std_logic;
			  cen : 			out std_logic;		-- Chip Enable
			  oen : 			out std_logic;		-- Output Enable
			  wen : 			out std_logic;		-- Write Enable
           busy : 		out std_logic);
end component;


-- component for initializing BT481 colormap
component bt481_init
port(
	reset		: in std_logic;
	clk			: in std_logic;
	load_color	: in std_logic;
	wrb			: out std_logic;
	rdb			: out std_logic;
	d			: out std_logic_vector(7 downto 0);
	rs			: out std_logic_vector(2 downto 0)
);
end component;

-- component for displaying RAM data on a VGA monitor
component bt481_vga
port(
	clk		: in std_logic;		-- VGA dot clock
	reset	: in std_logic;		-- asynchronous reset
	hsyncb	: out std_logic;	-- horizontal (line) sync
	vsyncb	: out std_logic;	-- vertical (frame) sync
	pixel	: out std_logic_vector(7 downto 0);		-- red,green,blue colors
	address	: out std_logic_vector(18 downto 0);	-- address into video RAM
	data	: in std_logic_vector(7 downto 0);		-- data from video RAM
	ceb		: out std_logic;	-- video RAM chip enable
	oeb		: out std_logic;	-- video RAM output enable
	web		: out std_logic;	-- video RAM write enable
	blankb	: out std_logic		-- video blanking
);
end component;

-- component for grabbing video data
component FrameLoader
Port ( 	  reset : in std_logic;								--asynch Reset (active high)
			  trig : in std_logic;								--start grabbing (active high)
	 		  llck : in std_logic;								--Clock from SAA7113
           vpo : in std_logic_vector(7 downto 0);		--Data from SAA7113
           rts : in std_logic_vector(1 downto 0);		--realtime video status
           ce : out std_logic;								--chip enable to video ram (active low)
           oe : out std_logic;								--outp enable to video ram (active low)
           we : out std_logic;								--write enable to video ram (active low)
           adr : out std_logic_vector(18 downto 0);	--address into video ram
           data : out std_logic_vector(15 downto 0);	--data into video ram
			  dataReady: out std_logic );						--High when entire fream read
end component;


--component saa7113init	-- initialization circuit for SAA7113
--	generic(
--		CLK_FREQ: integer
--	);
--	port(
--		reset : in std_logic;
--		clk : in std_logic;
--		scl_in : in std_logic;
--		scl_out : out std_logic;
--		sda_in : in std_logic;
--		sda_out : out std_logic
--	);
--end component;

component ibuf
	port(
		I: in std_logic;
		O: out std_logic
	);
end component;

component bufg
	port(
		I: in std_logic;
		O: out std_logic
	);
end component;


component pullup		-- pullup component
	port(
		O: out std_logic
	);
end component;

component IOBUF_F_12	-- tristate I/O buffer component
	port (
		I : in STD_LOGIC;
		T : in STD_LOGIC;
		O : out STD_LOGIC;
		IO : inout STD_LOGIC
	);
end component;

type STATE_TYPE_DOWN is (
			DOWN_IDLE,
			DOWN_H_TRIG,
			DOWN_H_TRANS,
			DOWN_WAIT_STATE1,
			DOWN_V_TRIG,
			DOWN_V_TRANS,
 		   DOWN_INC_TRANS
			);

-- signals
--*****************************************************************
signal cnt: std_logic_vector(15 downto 0);
signal slow_clk: std_logic; 		-- clk with appr. 400Hz
signal reset,resetn: std_logic;
signal trigger: std_logic;
signal trigStore1: std_logic_vector(2 downto 0);
signal ce_frl: std_logic;					--chip enable to video ram (active low)
signal oe_frl: std_logic;					--outp enable to video ram (active low)
signal we_frl: std_logic;					--write enable to video ram (active low)
signal adr_frl: std_logic_vector(18 downto 0);					--address into video ram
signal data_frl: std_logic_vector(15 downto 0);				--data into video ram
signal dataReady: std_logic;				--High when entire fream read
signal ce_vga: std_logic;
signal oe_vga: std_logic;
signal we_vga: std_logic;
signal adr_vga: std_logic_vector(18 downto 0);
signal data_vga: std_logic_vector(7 downto 0);
signal gnd_net	: std_logic;		-- logic-0 net
signal sda_out, sda_in, scl_out, scl_in : std_logic;	-- internal I2C interface signals
signal LOW, HIGH : std_logic;  -- Constatnt LOW and HIGH
signal iclk,bclk : std_logic;  -- Input and buffered clock
signal mainState: std_logic_vector(2 downto 0);
signal mainState2: std_logic_vector(3 downto 0);

-- for wavelettransform
signal h_size_wvt,v_size_wvt : std_logic_vector(9 downto 0);
signal verOffset_wvt,horOffset_wvt : std_logic_vector(9 downto 0);
signal line_adr, pix_adr: std_logic_vector(9 downto 0);
signal data2wvt : std_logic_vector(7 downto 0);
signal data_from_wvt : std_logic_vector(7 downto 0);
signal wr_wvt_data : std_logic;
signal ce_wvt: std_logic;
signal oe_wvt: std_logic;
signal we_wvt: std_logic;
signal busy_wvt,trig_wvt,trig_wvt2: std_logic;
signal trigStore: std_logic_vector(2 downto 0);

-- for Statemachine vertical horizontal transform
signal stateclk: std_logic;
signal nSdown,cSdown : STATE_TYPE_DOWN;
signal adr_hor,adr_vert,adr_wvt : std_logic_vector(18 downto 0);
signal v_trans_wvt_busy : std_logic;
signal h_size_2,v_size_2 : std_logic_vector(9 downto 0);
signal incLevel : std_logic;
signal level: std_logic_vector(1 downto 0);

-- for USB transmission
signal adr_trans: std_logic_vector(18 downto 0);
signal data_trans : std_logic_vector(7 downto 0);
signal ce_trans: std_logic;
signal oe_trans: std_logic;
signal we_trans: std_logic;
signal busy_trans,trig_transmit: std_logic;
signal trig_StoreTrans: std_logic_vector(2 downto 0);


begin



-- ____________________________________________________________________
-- ====================================================================
-- Different signal and net declarations
-- ====================================================================

	LOW <= LO;
	HIGH <= HI;


-- When using FPGA Express the following lines must be uncommented
--	ui: ibuf port map(I=>clk, O=>iclk);
--	ub: bufg port map(I=>iclk,O=>bclk);
	bclk <= clk;

	rd(8)<=llck;		-- clk for logic Analyzer
	rd(9)<=bclk;		-- clk for logic Analyzer
	rd(10)<=busy_wvt;		-- clk for logic Analyzer
	rd(11)<=busy_trans;		-- clk for logic Analyzer

	rceb <= HI;		-- disable right-bank of RAM that shares pixel data bus
	trste <= HI;	-- disable Ethernet PHY chip data pins

--	reset <= lp_d(2);	-- connect reset signal to D2 pin of parallel port
	process(sw1,sw2)
	begin
		if (sw1=LO and sw2=LO) then
			reset <= HI;
		else
			reset <= LO;
		end if;
	end process;
--	reset <= not sw1;	-- push-button 1 controls reset for the logic circuit
	resetn <= not reset;





	-- debounce for trigger Frameload
	process (slow_clk,reset)
	begin
		if slow_clk'event and slow_clk=HI then
			if reset=LO then
				trigStore1(0) <= not sw1;
				trigStore1(1) <= trigStore1(0);
				trigStore1(2) <= trigStore1(1);
			end if;
		end if;
	end process;
	trigger <= HI when (trigStore1 = "001") else LO;



	-- debounce for trigger wvt Transform
	process (slow_clk,reset)
	begin
		if slow_clk'event and slow_clk=HI then
			if reset=LO then
				trigStore(0) <= not sw2;
				trigStore(1) <= trigStore(0);
				trigStore(2) <= trigStore(1);
			end if;
		end if;
	end process;
	trig_wvt <= HI when (trigStore = "001") else LO;



	-- debounce for trigger sending frame over USB
	process (slow_clk,reset)
	begin
		if slow_clk'event and slow_clk=HI then
			if reset=LO then
				trig_StoreTrans(0) <= not sw4;
				trig_StoreTrans(1) <= trig_StoreTrans(0);
				trig_StoreTrans(2) <= trig_StoreTrans(1);
			end if;
		end if;
	end process;
	trig_transmit <= HI when (trig_StoreTrans = "001") else LO;



	-- divide the 50 MHz input clock down to some hundred Hz
	process(bclk)
	begin
		if (bclk'event and bclk=HI) then
			cnt <= cnt + 1;
		end if;
	end process;

	slow_clk <= cnt(14);














-- ____________________________________________________________________
-- ====================================================================
-- Controls and MUX for the RAM Signals
-- ====================================================================

	mainState <= (not dataReady & busy_wvt & busy_trans);
	mainState2 <= (not dataReady & busy_wvt & wr_wvt_data & busy_trans);


	-- with mainState select source or sink for video ram
	with mainState select
			la(18 downto 0) 	<= adr_frl when  "100",
									   adr_wvt when  "010",
									   adr_trans when "001",
									   adr_vga when others;
	with mainState2 select
			ld(15 downto 0) 	<= ("00000000"&data_frl(7 downto 0)) when "1000",
										(others => 'Z') when "0100",
										("00000000"&data_from_wvt)   when "0110",
										(others => 'Z') when "0001",
										(others => 'Z') when others;

	with mainState select
			lceb 					<= ce_frl when "100",
										ce_wvt when "010",
										ce_trans when "001",
										ce_vga when others;

	with mainState select
			loeb					<= oe_frl when  "100",
										oe_wvt when  "010",
										oe_trans when "001",
										oe_vga when others;

	with mainState select
			lweb					<= we_frl when  "100",
										we_wvt when  "010",
										we_trans when "001",
										we_vga when others;

	with mainState select
			data_vga				<= (others => '0') when "100",
										(others => '0') when "010",
										(others => '0') when "001",
										ld(7 downto 0) when others;

	with mainState select
			data2wvt				<= ld(7 downto 0) 	when "010",
										(others => '0') 	when others;

	with mainState select
			data_trans			<= ld(7 downto 0)		when "001",
										(others => '0') 	when others;










-- ____________________________________________________________________
-- ====================================================================
-- Initialization of SAA7113
-- ====================================================================


-- parallel port I2C interface 
	scl_out <= lp_d(0);
	sda_out <= lp_d(1);
--	lp_s(3) <= sda_in;
	lp_s <= sda_in;

	-- connect I2C pins of the SAA7113 initializer to the FPGA pins.
	-- I2C pins are configured as open-collector with pullups.
	gnd_net <= '0';
	-- connect pullup to I2C clock pin
	u0: pullup port map(O=>scl);
	-- tristate buffer used as open-collector buffer for I2C clock signal
	u1: IOBUF_F_12 port map(
		I => gnd_net,	-- buffer input is grounded
		T => scl_out,	-- clock output from component drives buffer tristate-enable
		O => scl_in,	-- level on pin goes back to component I2C clock input
		IO => scl		-- buffer output connects to pin
	);
	-- connect pullup to I2C data pin
	u2: pullup port map(O=>sda);
	-- tristate buffer used as open-collector buffer for I2C data signal
	u3: IOBUF_F_12 port map(
		I => gnd_net,	-- buffer input is grounded
		T => sda_out,	-- data output from component drives buffer tristate-enable
		O => sda_in,	-- level on pin goes back to component I2C data input
		IO => sda		-- buffer output connects to pin
	);

	-- instantiate component to initialize the SAA7113 video decoder chip
--	init_videoin: saa7113init
--		generic map(
--			CLK_FREQ => 50_000	-- clock frequency to Virtex should be 50 MHz
--		)
--		port map(
--			reset => reset,		-- reset input
--			clk => bclk,			-- main clock input
--			scl_in => scl_in,	-- I2C clock from FPGA pin to component
--			scl_out => scl_out,	-- I2C clock from component to FPGA pin
--			sda_in => sda_in,	-- I2C data from FPGA pin to component
--			sda_out => sda_out	-- I2C data from component to FPGA pin
--		);









-- ____________________________________________________________________
-- ====================================================================
-- Component for Reading data from SAA 7113
-- ====================================================================


	-- component for grabbing video data
	saa7113: FrameLoader
	port map ( 	 
			  reset 	=> reset,				--asynch Reset (active high)
			  trig	=> trigger,				--start grabbing (active high)
	 		  llck 	=> llck,					--Clock from SAA7113
           vpo 	=> vpo, 					--Data from SAA7113
           rts 	=> rts,					--realtime video status
           ce 		=> ce_frl,				--chip enable to video ram (active low)
           oe 		=>	oe_frl,				--outp enable to video ram (active low)
           we 		=> we_frl,				--write enable to video ram (active low)
           adr 	=> adr_frl,				--address into video ram
           data 	=> data_frl,			--data into video ram
			  dataReady => dataReady		--High when entire fream read
			  );













-- ____________________________________________________________________
-- ====================================================================
-- Wavelet Transformation 
-- ====================================================================

	-- Statemachine for vertical horizontal Transform
	stateclk <= cnt(2);
	process(stateclk,reset)
	begin
		if (reset=HI) then
			cSdown <= DOWN_IDLE;
		elsif stateclk'event and stateclk=HI then
			cSdown <= nSdown;
		end if;
	end process;

	process(reset,trig_wvt,cSdown)
	begin
			trig_wvt2 <= LO;
			v_trans_wvt_busy <= LO;
			incLevel <= LO;
			case cSdown is
				
			when DOWN_IDLE =>
				if trig_wvt=HI then
					nSdown <= DOWN_H_TRIG;
				else
					nSdown <= DOWN_IDLE;
				end if;

		  	when DOWN_H_TRIG =>
				trig_wvt2 <= HI;
				nSdown <= DOWN_H_TRANS;

			when DOWN_H_TRANS =>
				if busy_wvt = LO then
					nSdown <= DOWN_WAIT_STATE1;
				else
					nSdown <= DOWN_H_TRANS;
				end if;

			when DOWN_WAIT_STATE1 =>
				v_trans_wvt_busy <= HI;
				nSdown <= DOWN_V_TRIG;

			when DOWN_V_TRIG =>
				v_trans_wvt_busy <= HI;
				trig_wvt2 <= HI;
				nSdown <= DOWN_V_TRANS;

			when DOWN_V_TRANS =>
				v_trans_wvt_busy <= HI;
				if busy_wvt = LO then
					nSdown <= DOWN_INC_TRANS;
				else
					nSdown <= DOWN_V_TRANS;
				end if;

			when DOWN_INC_TRANS =>
					incLevel <= HI;
					nSdown <= DOWN_IDLE;

			when others =>
				nSdown <= DOWN_IDLE;
		  	end case;
	end process;


	
	adr_hor	<= (line_adr(8 downto 0) & pix_adr(9 downto 0));
	adr_vert <= (pix_adr(8 downto 0) & line_adr(9 downto 0));

	-- MUX between vert. and hor. transform
	h_size_wvt <= v_size_2 when (v_trans_wvt_busy=HI) else h_size_2;
	v_size_wvt <= h_size_2 when (v_trans_wvt_busy=HI) else v_size_2;
	verOffset_wvt <= horOffset when (v_trans_wvt_busy=HI) else verOffset;
	horOffset_wvt <= verOffset when (v_trans_wvt_busy=HI) else horOffset;	
	adr_wvt <= adr_vert when (v_trans_wvt_busy=HI) else adr_hor;


	-- Wavelettransformation Component
	wvt: wt_1Down_8
    port map ( 
	 		  resetn 	=> resetn,
           clk  		=> cnt(1),
           trigger  	=> trig_wvt2,
			  horOffset => horOffset_wvt,
			  verOffset => verOffset_wvt,
			  h_size		=>	h_size_wvt,	
			  v_size		=>	v_size_wvt,
           pix_adr 	=>	pix_adr,
			  line_adr	=>	line_adr,
           data_in	=> data2wvt,
           data_out	=>	data_from_wvt,
			  write_data=> wr_wvt_data,
			  cen			=>	ce_wvt,
			  oen 		=>	oe_wvt,
			  wen 		=>	we_wvt,
           busy 		=> busy_wvt
	);










-- ____________________________________________________________________
-- ====================================================================
-- Select the Picture Size depending on the current Level of 
-- transformation
-- ====================================================================

	-- counter for level
	process (stateclk, reset, incLevel,dataReady)
	begin
		if ((reset=HI) or (dataReady=LO)) then
			level <= "00";
		elsif stateclk'event and stateclk=HI then
			if incLevel = HI then
				level <= level + 1;
			end if;
		end if;
	end process;	


	process (level)
	begin
		case level is
			when "00" =>
				h_size_2 <= h_size;
				v_size_2 <= v_size;
			when "01" =>
				h_size_2 <= ("0" & h_size(9 downto 1));
				v_size_2 <= ("0" & v_size(9 downto 1));
			when "10" =>
				h_size_2 <= ("00" & h_size(9 downto 2));
				v_size_2 <= ("00" & v_size(9 downto 2));
			when "11" =>
				h_size_2 <= ("000" & h_size(9 downto 3));
				v_size_2 <= ("000" & v_size(9 downto 3));
			when others =>
				h_size_2 <= ("0000" & h_size(9 downto 4));
				v_size_2 <= ("0000" & v_size(9 downto 4));
		end case;
	end process;




-- ____________________________________________________________________
-- ====================================================================
-- USB communication
-- ====================================================================
trans: transmit_frame port map ( 
				reset 		=> reset,
				clk 			=> clk,
				trigger 		=> trig_transmit,
				h_size		=> h_size,
				v_size		=> v_size,
				h_offset		=> horOffset,
				v_offset		=> verOffset,
				data 			=> data_trans,
				adr 			=> adr_trans,
				ce 			=> ce_trans,
				oe 			=> oe_trans,
				we 			=> we_trans,
				VPO 			=> usbvpo,
				FSE 			=> usbvmo,
				SUSPND 		=> usbsuspnd,
				OEn 			=> usboeb,
				busy 			=> busy_trans
	);





-- ____________________________________________________________________
-- ====================================================================
-- Init for VGA RAMDAC
-- ====================================================================


	-- initialize the BT481 colormap
	ramdac_init: bt481_init
	port map(
			reset			=> reset,
			clk			=> cnt(1),	-- run initializer off 12.5 MHz clock
			load_color 	=> LOW,	-- DIP switch 1 selects between colorful and gray palettes
			wrb			=> wrb,	-- write-strobe of BT481
			rdb			=> rdb,	-- read-strobe of BT481
			d				=> d,	-- colormap register data bus to BT481
			rs				=> rs	-- register address bus to BT481
	);








-- ____________________________________________________________________
-- ====================================================================
-- VGA OUTPUT
-- ====================================================================

	-- VGA generator gets data from the left-bank of RAM and sends it to the
	-- BT481 where it is displayed on a VGA monitor

	pixelclk <= cnt(0);			-- BT481 gets a new pixel at 25 MHz rate

	vga_gen: bt481_vga
		port map(
			clk		=> cnt(0),	-- VGA clock is 25 MHz
			reset	=> reset,
			hsyncb	=> hsyncb,	-- direct output to VGA horizontal sync
			vsyncb	=> vsyncb,	-- direct output to VGA vertical sync
			pixel	=> p,		-- pixel bus to BT481
			address	=> adr_vga,	-- address bus to left-bank of RAM
			data	=> data_vga,	-- data bus from left-bank of RAM
			ceb		=> ce_vga,	-- chip-enable for left-bank of RAM
			oeb		=> oe_vga,	-- output-enable for left-bank of RAM
			web		=> we_vga,	-- write-enable for left-bank of RAM
			blankb	=> blankb	-- blanking control to BT481
		);



end behavioral;
