library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity write_test_data is
    Port ( reset 			: in std_logic;
           clk 			: in std_logic;
           trigger 		: in std_logic;
           data_out 		: out std_logic_vector(7 downto 0);
           adr 			: out std_logic_vector(18 downto 0);
           cen 			: out std_logic;
           oen 			: out std_logic;
           wen 			: out std_logic;
           busy 			: out std_logic);
end write_test_data;

architecture behavioral of write_test_data is

constant NO  : std_logic := '0';
constant YES : std_logic := '1';
constant LO  : std_logic := '0';
constant HI  : std_logic := '1';

-- ROM with test data 
constant ROM_SIZE		: integer := 10;	-- initialize 98/2=49 registers
--constant END_ADDRESS	: integer := ROM_SIZE-1;
subtype rom_word is std_logic_vector(7 downto 0);
type rom_table is array (0 to 15) of rom_word;
constant TEST_DATA : rom_table := (
	rom_word'("11111111"),		-- 255 (-1)
	rom_word'("11110000"), 		-- 240 (-16)
	rom_word'("00000000"), 		-- 0
	rom_word'("00000000"), 		-- 0
	rom_word'("00011110"), 		-- 30
	rom_word'("00110010"), 		-- 50
	rom_word'("11101011"), 		-- 235 (-21)
	rom_word'("11110111"), 		-- 247 (-9)
	rom_word'("11110111"), 		-- 247 (-9)
	rom_word'("11111111"),		-- 255 (-1)
	rom_word'("00000000"),		-- END
	rom_word'("00000000"),		-- END
	rom_word'("00000000"),		-- END
	rom_word'("00000000"),		-- END
	rom_word'("00000000"),		-- END
	rom_word'("00000000")		-- END
);

constant LINE_ADR : std_logic_vector(8 downto 0) := "000000100";

type STATE_TYPE is (
		IDLE,
		WRITE_1,
		WRITE_2,
		INC_PIX
);

  subtype addr_counter is integer range 0 to ROM_SIZE;
  signal adress			: addr_counter;	-- address into the ROM with initialization data

signal pix_adr : std_logic_vector(3 downto 0);
signal currState, nextState : STATE_TYPE;
signal fin : std_logic;



begin

	adr <= (LINE_ADR & "000000" & pix_adr);
   cen <= LO;	-- always enable RAM
   oen <= HI;	-- always disable RAM output

	data_out <= TEST_DATA(adress);

	-- Detect End of Testdata
	process (pix_adr)
	begin
		if pix_adr >= ROM_SIZE then
			fin <= HI;
		else
			fin <= LO;
		end if;
	end process;

	-- Counter for Pixel
	process (reset, clk, currState)
	begin
		if reset = HI then
			pix_adr <= (others => '0');
			adress <= 0;
		elsif clk'event and clk = HI then
			if currState = IDLE then
				pix_adr <= (others => '0');
				adress <= 0;
			elsif currState = INC_PIX then
				pix_adr <= pix_adr + 1;
				adress <= adress + 1;
			end if;
		end if;
	end process;



	-- Statemachine for write
	process (clk, reset)
	begin
		if (reset = HI) then
			currState <= IDLE;
		elsif clk'event and clk=HI then
			currState <= nextState;
		end if;
	end process;


	process (currState, trigger, fin)
	begin
		busy <= HI;
		wen <= HI;

		case currState is
		
		when IDLE =>
			busy <= LO;
			if trigger = HI then
				nextState <= WRITE_1;
			else
				nextState <= IDLE;
			end if;


		when WRITE_1 =>
			nextState <= WRITE_2;

		when WRITE_2 =>
			wen <= LO;
			nextState <= INC_PIX;

		when INC_PIX =>
			if fin = HI then
				nextState <= IDLE;
			else
				nextState <= WRITE_1;
			end if;

		when others =>
			nextState <= IDLE;
		end case;
			
	end process;

end behavioral;
