library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity wt_1level is
    Port ( resetn : 		in std_logic;
           clk : 			in std_logic;
           trigger : 	in std_logic;
			  horOffset : 	in	 std_logic_vector(9 downto 0); 	--Picture Offset in RAM
			  verOffset : 	in	 std_logic_vector(9 downto 0); 	--Picture Offset in RAM
			  h_size:		in  std_logic_vector(9 downto 0);	-- horizontal Picture Size (>16)
			  v_size:		in  std_logic_vector(9 downto 0);	-- vertical Picture Size
           pix_adr: 		out std_logic_vector(9 downto 0);
			  line_adr:		out std_logic_vector(9 downto 0);
           data_in : 	in  std_logic_vector(15 downto 0);
           data_out : 	out std_logic_vector(15 downto 0);
			  write_data : out std_logic;
			  cen : 			out std_logic;		-- Chip Enable
			  oen : 			out std_logic;		-- Output Enable
			  wen : 			out std_logic;		-- Write Enable
           busy : 		out std_logic);
end wt_1level;

architecture behavioral of wt_1level is

-- ====================================================================
-- Architecture declarations
-- ====================================================================


-- wavelettransformation Core
-----------------------------------------------------------------------
component CD53Level1_8v1
	port(
		enableN, clk, reset, clk_en		: IN	STD_LOGIC;
		data										: IN	STD_LOGIC_VECTOR(7 downto 0);
		dout										: OUT	STD_LOGIC_VECTOR(7 downto 0);
		cout										: OUT	STD_LOGIC_VECTOR(7 downto 0);
		clk_en_out,enableOutN				: OUT	STD_LOGIC);
end component;
-- old Component
--component CD53Level1_16v3
--	port(
--		horSync, clk, reset, clk_en	: IN	STD_LOGIC;
--		data									: IN	STD_LOGIC_VECTOR(15 downto 0);
--		dout									: OUT	STD_LOGIC_VECTOR(15 downto 0);
--		cout									: OUT	STD_LOGIC_VECTOR(15 downto 0);
--		clkEn,horSyncOut					: OUT	STD_LOGIC);
--end component;



-----------------------------------------------------------------------
-- Constants:
-----------------------------------------------------------------------
constant HI : std_logic := '1';
constant LO : std_logic := '0';
constant YES : std_logic := '1';
constant NO : std_logic := '0';



-----------------------------------------------------------------------
-- States for Wavelettransform Statemachine
-----------------------------------------------------------------------
type STATE_TYPE is (
			IDLE,					-- 000000001
		 	INC_LINE,			-- 000000010
		 	TRANSF_LINE,		-- 000000100
		 	RD_PIX,				-- 000001000
			WR1_d0,				-- 000010000
			WR2_d0,				-- 000100000
			WR1_a0,				-- 001000000
			WR2_a0,				-- 010000000
			GEN_CLKEN);			-- 100000000
--Remove these two lines to allow Express to determine the encoding
attribute ENUM_ENCODING: STRING;
attribute ENUM_ENCODING of STATE_TYPE: type is "000000001 000000010 000000100 000001000 000010000 000100000 001000000 010000000 100000000";

signal currState, nextState: STATE_TYPE; -- states for main statemachine
signal d0L,a0L : std_logic;	-- Dataready signals from wavelettransformers (Latched)
signal d0R,a0R : std_logic;	-- Dataready signals wavelettransformers (Reset)
signal clk_en : std_logic; -- Enable for clk wvt Level 0
signal dontread : std_logic;	-- signals not to Read data from memory
signal write_1, write_2 : std_logic;	-- signals to write data to memory
signal readFinished : std_logic; -- YES when entire Line is read
signal writeFinished: std_logic; -- YES when entire Line is written
signal pictureFinished: std_logic; -- YES when entire Picture is transformed
signal rLine: std_logic_vector(9 downto 0);	-- Line number that will be read
signal wLine: std_logic_vector(9 downto 0);	-- Line number that will be written
signal rPix:  std_logic_vector(9 downto 0);	-- Pixel number that will be read
signal d0Pix:  std_logic_vector(9 downto 0);	-- Pixel number detailcoef L0
signal a0Pix:  std_logic_vector(9 downto 0);	-- Pixel number appr.coef L2
signal pix_nr: std_logic_vector(9 downto 0);	-- Multiplexed pixelNr
signal pix_nr2: std_logic_vector(9 downto 0);	-- pixelNr with Offset added
signal line_adr2: std_logic_vector(9 downto 0);	-- lineAdr with Offset added
signal data_out2wvt: std_logic_vector(7 downto 0);	--Data from RAM to wvt Level0
signal data_wvt: std_logic_vector(15 downto 0); -- Data from wvt to RAM
signal switch_dataout : bit;	-- Control signal for dataout switch
signal data_wvt_mux: std_logic_vector(7 downto 0); -- Data from wvt muxed
signal reset: std_logic; 	-- is /resetn
-- Level 0 transformation
signal enable_L0,horSync: std_logic;	-- Enable for Level 0 wvt
signal dout_L0: std_logic_vector(7 downto 0); -- detail coeff from L0
signal cout_L0: std_logic_vector(7 downto 0); -- approx. coeff from L0
signal dout_L0_L: std_logic_vector(7 downto 0); -- detail coeff from L0
signal cout_L0_L: std_logic_vector(7 downto 0); -- approx. coeff from L0
signal clkEn_L0:	std_logic;	-- clk Enable for Level 1
signal hSyncOut_L0:	std_logic; -- Enable for Level 1



begin

-- ====================================================================
-- Architecture body
-- ====================================================================



-- wavelettransformation Cores
-----------------------------------------------------------------------
wvt_L0: CD53Level1_8v1 port map (
		enableN			=> horSync,
		clk				=> clk,
		reset				=> reset,
		clk_en			=> clk_en,
		data				=> data_out2wvt,
		dout				=> dout_L0,
		cout				=> cout_L0,
		clk_en_out		=> clkEn_L0,
		enableOutN		=> hSyncOut_L0
	);




-- ====================================================================
-- Combinational signals
-- ====================================================================

	reset <= not resetn;


-- read Finished Signal
-----------------------------------------------------------------------
	process (rPix)
	begin
		if rPix >= h_size then
			readFinished <= YES;
		else
			readFinished <= NO;
		end if;
	end process;

-- Picture Finished Signal
-----------------------------------------------------------------------
	process (wLine,readFinished,writeFinished)
	begin
		if (rLine >= v_size) and readFinished = YES and writeFinished = YES then
			pictureFinished <= YES;
		else
			pictureFinished <= NO;
		end if;
	end process;

-- Write Finished Signal
-----------------------------------------------------------------------
	process (d0Pix,a0Pix)
	begin
		if (d0Pix >= h_size) and 
		 	(a0Pix >= ("0" & h_size(9 downto 1))) then

			writeFinished <= YES;
		else
			writeFinished <= NO;
		end if;
	end process;



-- ====================================================================
-- Implementation of specific structures
-- ====================================================================

-- dontread Signal Latch
	process (clk, resetn, currState)
	begin
		if (resetn = LO) then
			dontread <= NO;
		elsif (clk'event and clk=HI) then
			if (currState=RD_PIX) or (currState=GEN_CLKEN) then
				dontread <= YES;
			else
				dontread <= NO;
		  	end if;
		end if;
	end process;

-- clk_en signal Latch
	process (clk, resetn)
	begin
		if resetn = LO then
			clk_en <= LO;
		elsif clk'event and clk=LO then
			clk_en <= dontread;
		end if;
	end process;

	
	process (clk,readFinished)
	begin
		if clk'event and clk=HI then
			if clk_en=HI and readFinished=YES then
				enable_L0 <= HI;
			elsif readFinished=NO then
				enable_L0 <= LO;
			end if;
		end if;
	end process;

	process (clk,readFinished)
	begin
		if readFinished=NO then
				horSync <= LO;			
		elsif clk'event and clk=HI then
			if enable_L0=HI then
				horSync <= HI;
			end if;
		end if;
	end process;




-- dataReady Signal Latches und WvtData Latches
--------------------------------------------------------------
	process (clk, resetn, d0R)
	begin
   	if (resetn=LO) then
			d0L <= NO;
		elsif (d0R = HI) then
			d0L <= NO;
      elsif (clk'event and clk = HI) then
			if (clkEn_L0 = HI) and (hSyncOut_L0 = LO) then
				d0L <= HI;
			end if;
		end if;
	end process;
	process (clk,clkEn_L0)
	begin
		if clk'event and clk=HI then
			if clkEn_L0 = HI then
				dout_L0_L <= dout_L0;
			else
				dout_L0_L <= dout_L0_L;
			end if;
		end if;
	end process;


	process (clk, resetn, a0R)
	begin
   	if (resetn=LO) then
			a0L <= NO;
		elsif (a0R = HI) then
			a0L <= NO;
      elsif (clk'event and clk = HI) then
			if (clkEn_L0 = HI) and (hSyncOut_L0 = LO) then
				a0L <= HI;
			end if;
		end if;
	end process;
	process (clk,clkEn_L0)
	begin
		if clk'event and clk=HI then
			if clkEn_L0 = HI then
				cout_L0_L <= cout_L0;
			else
				cout_L0_L <= cout_L0_L;
			end if;
		end if;
	end process;


-- ====================================================================
-- Adresscounters
-- ====================================================================

-- Counter for Lines
-----------------------------------------------------------------------
	process (clk, resetn)
	begin
   	if (resetn=LO) then
			rLine <= "0000000001";
			wLine <= (others => '0');
		elsif (clk'event and clk=HI) then

			if currState=IDLE then 
			-- Linecounter reset after picture is Transformed
				rLine <= "0000000001";
				wLine <= (others => '0');

			elsif currState=INC_LINE then
			-- Increment lincounter in inc_line State
				rLine <= rLine + 1;
				wLine <= wLine + 1;
			end if;
	  	end if;
	end process;


-- Counter for readPixel
-----------------------------------------------------------------------
	process (clk, resetn)
	begin
   	if (resetn=LO) then
			rPix <= (others => '0');
		elsif (clk'event and clk=HI) then

			if (currState=INC_LINE) or (currState=IDLE) then 
			-- Pixelcounter reset after line is transformed
				rPix <= (others => '0');

			elsif (currState=RD_PIX) then
			-- Increment pixelcounter after read has been performed
				rPix <= rPix + 1;
			end if;
	  	end if;
	end process;
				
				
-- Counter for detail coeff L0
-----------------------------------------------------------------------
	process (clk, resetn, currState)
	begin
   	if (resetn=LO) then
			d0Pix <= ("0" & h_size(9 downto 1));	-- h_size / 2
		elsif (clk'event and clk=HI) then

			if (currState=INC_LINE) or (currState=IDLE) then 
			-- Pixelcounter reset after line is transformed
				d0Pix <= ("0" & h_size(9 downto 1));	-- h_size / 2

			elsif (currState=WR2_d0) then
			-- Increment pixelcounter after write has been performed
				d0Pix <= d0Pix + 1;
			end if;
	  	end if;
	end process;



-- Counter for approximation coeff L2
-----------------------------------------------------------------------
	process (clk, resetn, currState)
	begin
   	if (resetn=LO) then
			a0Pix <= (others => '0');
		elsif (clk'event and clk=HI) then

			if (currState=INC_LINE) or (currState=IDLE) then 
			-- Pixelcounter reset after line is transformed
				a0Pix <= (others => '0');

			elsif (currState=WR2_a0) then
			-- Increment pixelcounter after write has been performed
				a0Pix <= a0Pix + 1;
			end if;
	  	end if;
	end process;


-- ====================================================================
-- MUX and Latch for Adress (Counters)
-- ====================================================================

	with nextState select
		pix_nr2 <= 	d0Pix		when WR1_d0,
						a0Pix		when WR1_a0,	
						rPix		when others;

		pix_nr <= horOffset	+ pix_nr2;

-- Latch the counters at the end of TRANSF_LINE State
-----------------------------------------------------------------------
	process (clk, currState)
	begin
		if clk'event and clk=HI then
			if currState = TRANSF_LINE then
				pix_adr <= pix_nr;					-- Pixeladress out
			end if;
		end if;
	end process;

	with currState select							-- LineAdress out
		line_adr2 <= 	rLine when RD_PIX,
							wLine when others;

	line_adr <= line_adr2 + verOffset;
	



-- ====================================================================
-- RAM Acces Signals
-- ====================================================================
	cen <= LO;	-- RAM always enabled

	with currState select	-- RAM output enable only for read and before read
		oen <= 	'0' when RD_PIX,
--					'0' when TRANSF_LINE,
					'1' when others;

	process(clk, write_1,write_2)
	begin
		if clk'event and clk=LO then
			wen <= not write_1;
		end if;
	end process;
			
-- data from RAM to wvt Level 1
-----------------------------------------------------------------------
	process(clk, currState)
	begin
		if clk'event and clk = HI then
			if currState = RD_PIX then
			 	data_out2wvt <= (data_in(7 downto 0));
			else
				data_out2wvt <= data_out2wvt;
			end if;
		end if;
	end process;



-- data from wvt to RAM
-----------------------------------------------------------------------
	with currState select
			write_data <= 	HI when WR1_d0,
								HI when WR2_d0,
								HI when WR1_a0,
								HI when WR2_a0,
								LO when others;

--	process(clk,write_1,write_2)
--	begin
--		if (clk=LO and	write_1 = YES) or (write_2 =YES) then
--			data <= data_wvt;
--		else
--			data <= (others => 'Z');
--		end if;
--	end process;



	
-- ====================================================================
-- MUX and Latch for Wavlettransformed Data
-- ====================================================================	
	with nextState select
		data_wvt_mux 	<= 	cout_L0_L		when WR1_a0,	
									dout_L0_L		when others;	
	
-- Latch the Wavlettransformed Data at the end of TRANSF_LINE State
-----------------------------------------------------------------------
	process (clk, currState)
	begin
		if clk'event and clk=HI then
			if currState = TRANSF_LINE then
				data_out <= ("00000000"&data_wvt_mux);
			end if;
		end if;
	end process;
	
	



-- ====================================================================
-- ====================================================================
--
-- Statemachine for wavelettransformation (read, write, send pixel to
--                                         transform cores)
--
-- ====================================================================
-- ====================================================================
	SYNC_PROC: process (clk, resetn)
   begin
   	if (resetn=LO) then
              currState <= IDLE;
      elsif (clk'event and clk = HI) then
            currState <= nextState;
       end if;
    end process;
 
   COMB_PROC: process (currState, d0L, a0L, trigger,
	 							dontread, readFinished, writeFinished,
								pictureFinished)
   begin
	busy <= YES;	-- always busy (except in Idle state)
	d0R	<= NO;	-- don't reset the dataready latches normaly
	a0R	<= NO;
	write_1 <= NO;
	write_2 <= NO;

	if resetn= LO then
		nextState <= IDLE;
		busy <= NO;	
		d0R	<= NO;	
		a0R	<= NO;
		write_1 <= NO;
		write_2 <= NO;
	else

		case currState is

			-- waiting to be triggered
			when IDLE =>
				busy <= NO;

	      	if ( trigger = YES ) then
					nextState <= TRANSF_LINE;
				else
					nextState <= IDLE;
	   	   end if;


			-- In this state the linecounter is incremented
			when INC_LINE =>
			 	if pictureFinished = YES then
					nextState <= IDLE;
				else
					nextState <= TRANSF_LINE;
				end if;
	
			-- this is a 'inbetween' state
			-- from here it goes to the different write and the read state
			when TRANSF_LINE =>
				if ( (readFinished = YES) and (writeFinished=YES)) then
					nextState <= INC_LINE;
	
				elsif ( (readFinished = NO) and (d0L=NO) and (a0L=NO) ) then
					nextState <= RD_PIX;
	
				elsif ( (readFinished = YES) and (d0L=NO) and (a0L=NO) ) then
					nextState <= GEN_CLKEN;

				elsif ( (writeFinished = NO)  and (d0L=YES) ) then
					nextState <= WR1_d0;

				elsif ( (writeFinished = NO)  and (d0L=NO)
							and (a0L=YES) ) then
					nextState <= WR1_a0;
				else
					nextState <= TRANSF_LINE;
				end if;

			-- read one pixel
			when RD_PIX =>
				nextState <= TRANSF_LINE;
	
			-- generate clk_en after reading has finished
			when GEN_CLKEN =>
				nextState <= TRANSF_LINE;
	

			-- write detailcoef Level 0
			when WR1_d0 =>
				write_1 <= YES;	-- signal writing to RAM
				nextState <= WR2_d0;
			when WR2_d0 =>
				write_2 <= YES;	-- signal writing to RAM
				d0R <= YES;		-- reset the dataReady Signal
				nextState <= TRANSF_LINE;
	
			-- write approx.coef Level 0
			when WR1_a0 =>
				write_1 <= YES;	-- signal writing to RAM
				nextState <= WR2_a0;
			when WR2_a0 =>
				write_2 <= YES;	-- signal writing to RAM
				a0R <= YES;		-- reset the dataReady Signal
				nextState <= TRANSF_LINE;
	
			when others =>
					nextState <= IDLE;
    
			end case;
		end if;
	end process;


end behavioral;

