library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;



--
-- I2C transmitter component
--
entity i2c_tx is

generic(
	NUM_OF_DATABITS	: integer	:= 8;
	CLK_FREQ		: integer	:= 50_000 -- input clock in KHz
);

port (
	reset	: in	std_logic; -- reset the transmitter
	clk		: in	std_logic; -- input clock
	data_in	: in	std_logic_vector(NUM_OF_DATABITS-1 downto 0); -- transmit data
	wr		: in	std_logic; -- write control (active-high on rising edge of clk)
	bsy		: out	std_logic; -- transmitter busy
	tx_failed : out	std_logic; -- transmitter failed (NACK or failed arbitration)
	sda_out	: out	std_logic; -- I2C data signal output
	sda_in	: in	std_logic; -- I2C data signal input
	scl_out	: out	std_logic; -- I2C clock signal output
	scl_in	: in	std_logic  -- I2C clock signal input
);

end i2c_tx;




architecture i2c_tx_arch of i2c_tx is

-- define a few constants
constant NO		: std_logic := '0';
constant YES	: std_logic := '1';
constant LO		: std_logic := '0';
constant HI		: std_logic := '1';

-- declare the transmit data shift register and the transmit bit counter
constant MSBIT		: integer := NUM_OF_DATABITS-1;
subtype bit_count is integer range 0 to NUM_OF_DATABITS;
signal tx_reg		: std_logic_vector(MSBIT downto 0);
signal bit_counter	: bit_count;

-- define some timing parameters for the I2C bus signals
-- (times are in nanoseconds and frequency is in KHz)
constant BUS_FREE_TIME		: integer := integer((CLK_FREQ * 4700) / 1E6 + 1);
constant START_HOLD_TIME	: integer := integer((CLK_FREQ * 4000) / 1E6 + 1);
constant DATA_SETUP_TIME	: integer := integer((CLK_FREQ * 250)  / 1E6 + 1);
constant DATA_HOLD_TIME		: integer := integer((CLK_FREQ * 300)  / 1E6 + 1);
constant CLOCK_LOW_TIME		: integer := integer((CLK_FREQ * 4700) / 1E6 + 1);
constant CLOCK_HIGH_TIME	: integer := integer((CLK_FREQ * 4000) / 1E6 + 1);
constant STOP_SETUP_TIME	: integer := integer((CLK_FREQ * 4000) / 1E6 + 1);
constant ABORT_TIME			: integer := integer((CLK_FREQ * 10000)/ 1E6 + 1);
subtype timer is integer range 0 to ABORT_TIME;
signal scl_timer, sda_timer : timer;

-- define the states of the I2C transmitter
type i2c_state is (
	idle,				-- waiting for something to transmit
	waiting_to_start,	-- waiting for I2C bus to be idle
	tx_start_condition,	-- sending a start signal on the bus
	set_data_bit,		-- output next data bit on bus
	pulse_clock,		-- pulse the clock
	hold_data_bit,		-- hold the data bit after the pulse
	tx_stop_condition1,	-- initiate a transmitter stop
	tx_stop_condition2, -- complete the transmitter stop
	abort				-- some problem occurred
);
signal i2c_next : i2c_state;

begin

	i2c_state_machine: process(reset,clk)
	begin

		if(reset = YES) then
			sda_out <= HI;		-- place SDA and SCL in HI state to free I2C bus
			scl_out <= HI;
			sda_timer <= 0;		-- reset I2C interval timers
			scl_timer <= 0;
			tx_reg <= (others=>HI);	-- fill transmit shift register with 1s
			bit_counter <= 0;	-- transmit bit counter is zero
			i2c_next <= idle;	-- start in idle state
			bsy <= NO;			-- transmitter not busy
			tx_failed <= NO;	-- clear transmission failure indicator

		elsif(clk'event and clk='1') then

			-- increment I2C interval timers for SDA and SCL signals
			sda_timer <= sda_timer + 1;
			scl_timer <= scl_timer + 1;

			case i2c_next is

			-- idle and waiting for some data to transmit
			when idle => -- idle; waiting for some data to transmit
				sda_timer <= 0;		-- reset I2C interval timers
				scl_timer <= 0;
				sda_out <= HI;	-- raise SDA and SCL to free I2C bus
				scl_out <= HI;
				bsy <= NO;		-- transmitter is not busy
				if(wr = YES) then
					-- input data is available
					tx_reg <= data_in;	-- store input data into shift register
					i2c_next <= waiting_to_start;	-- exit the idle state
				end if;

			-- data register has bits to transmit, so wait for bus to become free
			when waiting_to_start => -- waiting for I2C bus to be free for transmission
				bsy <= YES;		-- indicate the transmitter is busy
				if(sda_in=HI and scl_in=HI) then
					-- data and clock are both high, so the bus is free
					if(sda_timer >= BUS_FREE_TIME) then
						-- the bus has been free long enough, so start transmission
						sda_out <= LO;	-- data goes low while clock is high to signal start condition
						sda_timer <= 0;	-- reset the SDA signal timer
						i2c_next <= tx_start_condition;	-- exit this state
					end if;
				else
					-- data and clock are not both high, so reset timer and wait
					-- for the bus to become free again
					sda_timer <= 0;
				end if;

			-- output start condition to begin data transmission
			when tx_start_condition =>
				bsy <= YES;		-- indicate the transmitter is busy
				if(sda_timer >= START_HOLD_TIME) then
					-- data has been low long enough, so start transmitting data bits
					bit_counter <= NUM_OF_DATABITS; -- init data bit counter
					sda_timer <= 0;	-- clear SDA and SCL timers
					scl_timer <= 0;
					scl_out <= LO; -- lower clock for start of data bits
					i2c_next <= set_data_bit;	-- exit this state
				end if;

			-- set current data bit level on sda while the clock is low
			when set_data_bit =>
				bsy <= YES;		-- indicate the transmitter is busy
				sda_out <= tx_reg(MSBIT); -- output most-significant bit of data
				if(sda_timer >= DATA_SETUP_TIME) then
					if(scl_timer >= CLOCK_LOW_TIME) then
						-- if data bit has been set up and clock has been low long
						-- enough, then raise clock to signal the receiver that it
						-- should accept the data bit.
						scl_out <= HI; -- raise clock
						if(scl_in = LO) then
							-- if something else on the bus is holding the clock low,
							-- then recycle and wait for the clock to go high
							scl_timer <= CLOCK_LOW_TIME;
							sda_timer <= DATA_SETUP_TIME;
						else
							-- ok, the clock is high so time out the high pulse
							scl_timer <= 0; -- reset the timers
							sda_timer <= 0;
							i2c_next <= pulse_clock;
						end if;
					end if;
				end if;

			-- hold data bit steady during the high part of the clock cycle
			when pulse_clock =>
				bsy <= YES;		-- indicate the transmitter is busy
				if(sda_in /= tx_reg(MSBIT) and bit_counter>0) then
					-- if the data bit doesn't match the value on the bus, then
					-- some contention has occurred.  Abort the transmission.
					tx_failed <= YES;
					scl_timer <= 0;
					i2c_next <= abort;
				elsif(sda_in /= LO and bit_counter=0) then
					-- if the bit counter is zero, then the receiver must be sending
					-- its acknowledge.  If the ack is not zero, then the receiver
					-- has not accepted the data and the transmission must be aborted.
					tx_failed <= YES;
					scl_timer <= 0;
					i2c_next <= abort;
				end if;
				if(scl_timer >= CLOCK_HIGH_TIME or scl_in=LO) then
					-- the high portion of the clock cycle is complete, so lower
					-- the clock and hold the data steady for a short time
					scl_out <= LO;
					scl_timer <= 0;
					sda_timer <= 0;
					i2c_next <= hold_data_bit;
				end if;

			-- hold the data steady for a while after the clock pulse ends
			when hold_data_bit =>
				bsy <= YES;		-- indicate the transmitter is busy
				if(sda_timer >= DATA_HOLD_TIME) then
					-- after the data hold time is met, decrement the counter for
					-- the data bits and shift the next data bit into the MSB of
					-- the data shift register.  1's are shifted into the register
					-- so a 1 will be output on sda when the data bits are exhausted
					-- and the acknowledge bit from the receiver appears.
					sda_timer <= 0;	-- reset SDA and SCL interval timers
					scl_timer <= 0;
					if(bit_counter /= 0) then
						-- still transmitting data bits
						tx_reg <= tx_reg(MSBIT-1 downto 0) & HI;	-- shift data
						bit_counter <= bit_counter - 1;	-- decrement data bit counter
						i2c_next <= set_data_bit;
					elsif(wr = YES) then
						-- done transmitting, but more data is available
						-- at the input port to the transmitter
						bsy <= NO;	-- indicate current transmission is done
						tx_reg <= data_in;	-- load next data word
						bit_counter <= NUM_OF_DATABITS; -- init data bit counter
						i2c_next <= set_data_bit;
					else
						-- done transmitting and no more data is available
						bsy <= NO;
						sda_out <= LO;	-- lower SDA to send stop signal
						i2c_next <= tx_stop_condition1;
					end if;
				end if;

			-- output stop condition to end data transmission
			when tx_stop_condition1 =>
				bsy <= NO;	-- transmitter is no longer busy sending data
				if(scl_timer >= CLOCK_LOW_TIME) then
					-- wait for clock to be low long enough before sending stop
					scl_timer <= 0;
					scl_out <= HI;	-- raise clock while data is low
					i2c_next <= tx_stop_condition2;
				end if;

			-- output stop condition to end data transmission
			when tx_stop_condition2 =>
				bsy <= NO;	-- transmitter is no longer busy sending data
				if(scl_timer >= STOP_SETUP_TIME) then
					-- wait for clock to be high long enough
					sda_out <= HI;	-- then raise data to signal stop
					i2c_next <= idle;
				end if;

			-- some problem occurred during transmission, so abort
			when abort =>
				tx_failed <= NO;	-- transmission failed
				bsy 	<= NO;		-- transmitter is no longer busy
				scl_out <= HI;		-- free the I2C bus
				sda_out <= HI;
				-- wait a while to let the I2C bus settle
				if(scl_timer >= ABORT_TIME) then
					i2c_next <= idle;
				end if;
			
			-- handle erroneous states
			when others =>
				i2c_next <= idle;

			end case;

		end if;

	end process;

end i2c_tx_arch;
